/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.initialisation;

import java.io.IOException;
import java.io.StringReader;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;


import org.jdom.Attribute;
import org.jdom.CDATA;
import org.jdom.Comment;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.EntityRef;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.ProcessingInstruction;
import org.jdom.Text;
import org.jdom.input.SAXBuilder;




public class InitialisationEngine 
{
    public InitialisationEngine() {
    }
    
    public HashMap[] intialiseQuestion(String QTIFile) 
    {
        HashMap arr[] = new HashMap [3];
        
        try 
        {
            SAXBuilder builder = new SAXBuilder();
            builder.setFeature("http://xml.org/sax/features/validation",false);
            builder.setFeature("http://apache.org/xml/features/validation/schema",false);
            //To set the schema location via porperties loader
            builder.setProperty("http://apache.org/xml/properties/schema/external-schemaLocation", "http://www.imsglobal.org/xsd/imsqti_v2p0 imsqti_v2p0.xsd");
            builder.setProperty("http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation","imsqti_v2p0.xsd");  
            
            Document doc = builder.build(new StringReader(QTIFile.trim()));
            Element root = doc.getRootElement();
            HashMap  responseVars = processResponseVars(root);
            HashMap  outcomeVars = processOutcomeVars(root);        
            HashMap templateVars = processTemplateVars(root);
            
            arr[0]=responseVars;
            arr[1]=outcomeVars;
            arr[2]=templateVars;
            
        }catch (JDOMException e) 
       { 
          System.out.println("QTI File is not valid.");
          System.out.println(e.getMessage());
          e.printStackTrace();
       }  
       catch (IOException e) 
       { 
          System.out.println("Could not check QTI File ");
          System.out.println(" because " + e.getMessage());
       }  
      return arr;
    }
    
    private HashMap processResponseVars(Element e)
    {
       HashMap respVars = new HashMap();
       try
       {
           System.out.println("Root Element is : "+e.getName());
           List list1 = e.getChildren() ;
            Iterator iterator1 = list1.iterator();
            System.out.println("Number of children in ROOT is : "+list1.size());
            while (iterator1.hasNext()) 
            {
                 Element child = (Element)iterator1.next();
                 System.out.println("Childern :"+child.getName()+":");
                 if(child.getName().equals("responseDeclaration")) 
                 {
                     String respDeclName;
                     HashMap respDecl = new HashMap();
                     respDeclName =child.getAttributeValue("identifier");
                     System.out.println("***Value of identifier :"+respDeclName);                     
                     //Add responseDeclaration attributes to HashMap
                     respDecl.put("identifier",child.getAttributeValue("identifier"));
                     String cardinality = (String)child.getAttributeValue("cardinality");
                     respDecl.put("cardinality",cardinality);
                     
                     respDecl.put("baseType",child.getAttributeValue("baseType"));
                     //Get the default Values of responseDeclaration tag
                      List list2 = child.getChildren();
                      Iterator iterator2 = list2.iterator();
                      System.out.println("Number of children for"+ child.getName()+" is : "+list2.size());
                      boolean defaultFlag=false;
                      boolean correctResponseFlag = false;
                      boolean mappingFlag=false;
                      boolean areaMappingFlag=false;
                      
                      while(iterator2.hasNext()) 
                      {
                             Element child1 = (Element) iterator2.next();
                             if(child1.getName().equals("defaultValue")) //defaultValue can occur only once
                             {
                                defaultFlag =true;
                                 respDecl.put("defaultValueInterpretation",child1.getAttributeValue("interpretation"));                                                           
                                 //Get all th values for defaultValue
                                 Vector res = getDefaultValues(child1);
                                 respDecl.put("defaultValue",res);
                             }
                             //Get the correct response Values
                             if(child1.getName().equals("correctResponse")) 
                             {
                                 correctResponseFlag=true;
                                 Vector correctResponseValues = getCorrectResponseValues(child1);
                                 respDecl.put("correctResponse",correctResponseValues); 
                             }                             
                             //Get the values of  mapping Tag .. mappingDefaultValue, mappingLowerBound,mappingUpperBound
                             if(child1.getName().equals("mapping")) 
                             {
                                 mappingFlag = true;
                                 respDecl.put("mappingDefaultValue",child1.getAttributeValue("defaultValue"));
                                 respDecl.put("mappingLowerBound",child1.getAttributeValue("lowerBound"));
                                 respDecl.put("mappingUpperBound",child1.getAttributeValue("upperBound"));
                                 //Get the mapEntry values
                                 HashMap mapRes = getMapEntryValues(child1);
                                 respDecl.put("mapEntry",mapRes);                                 
                             }
                          //Get the values of  areaMapping Tag .. areaMappingDefaultValue, areaMappingLowerBound,areaMappingUpperBound
                          if(child1.getName().equals("areaMapping")) 
                          {
                              areaMappingFlag = true;
                              respDecl.put("areaMappingDefaultValue",child1.getAttributeValue("defaultValue"));
                              respDecl.put("areaMappingLowerBound",child1.getAttributeValue("lowerBound"));
                              respDecl.put("areaMappingUpperBound",child1.getAttributeValue("upperBound"));
                              //Get the areaMapEntry values
                              ArrayList areaMapRes = getAreaMapEntryValues(child1);
                              respDecl.put("areaMapEntry",areaMapRes);                                 
                          }
                             
                      }
                      if((list2.size()==0)||(defaultFlag==false))
                      {
                          respDecl.put("defaultValueInterpretation",null);
                          respDecl.put("defaultValue",null);
                      }
                     //add candidate values
                      //TODO ResponseDeclaration defaultValue set here
                     // respDecl.put("candidateValues",respDecl.get("defaultValue"));   
                     //add the responseDeclaration to response variables
                     respVars.put(respDeclName,respDecl);
                     
                 }            
            }           
           
       }catch (Exception exc) 
       { 
          System.out.println("QTI File is not valid.");
          System.out.println(exc.getMessage());
          exc.printStackTrace();
       }  
       
       return respVars;
    }

    private HashMap processOutcomeVars(Element e)
    {
        HashMap outVars = new HashMap();
        
        try
        {
            System.out.println("Root Element is : "+e.getName());
            List list1 = e.getChildren() ;
             Iterator iterator1 = list1.iterator();
             System.out.println("Number of children in ROOT is : "+list1.size());
             while (iterator1.hasNext()) 
             {
                  Element child = (Element)iterator1.next();                  
                  if(child.getName().equals("outcomeDeclaration")) 
                  {
                      String outDeclName;
                      HashMap outDecl = new HashMap();
                      outDeclName =child.getAttributeValue("identifier");
                      System.out.println("***Value of identifier :"+outDeclName);
                      //Add responseDeclaration attributes to HashMap
                      outDecl.put("identifier",child.getAttributeValue("identifier"));
                      
                      outDecl.put("cardinality",child.getAttributeValue("cardinality"));
                      outDecl.put("baseType",child.getAttributeValue("baseType"));
                      outDecl.put("interpretation",child.getAttributeValue("interpretation"));
                      outDecl.put("longInterpretation",child.getAttributeValue("longInterpretation"));
                      outDecl.put("normalMaximum",child.getAttributeValue("normalMaximum"));   
                      //Get the default Values of outcomeDeclaration tag
                       List list2 = child.getChildren();
                       Iterator iterator2 = list2.iterator();
                       System.out.println("Number of children for"+ child.getName()+" is : "+list2.size());
                       while(iterator2.hasNext()) 
                       {
                              Element child1 = (Element) iterator2.next();
                              if(child1.getName().equals("defaultValue")) //defaultValue can occur only once
                              {
                                  outDecl.put("defaultValueInterpretation",child1.getAttributeValue("interpretation"));                                                           
                                  //Get all th values for defaultValue
                                  Vector res = getDefaultValues(child1);
                                  outDecl.put("defaultValue",res);
                              }
                       }
                       if(list2.size()==0) 
                       {
                           outDecl.put("defaultValueInterpretation",null);
                           outDecl.put("defaultValue",null);
                       }
                       
                       outDecl.put(child.getAttributeValue("identifier"),outDecl.get("defaultValue"));
                       
                      //add the responseDeclaration to response variables
                      outVars.put(outDeclName,outDecl);   
                  }
             }
        }catch (Exception exc) 
       { 
          System.out.println("QTI File is not valid.");
          System.out.println(exc.getMessage());
          exc.printStackTrace();
       }       
        
        /* Initialise the special variable completionStatus. 
         * 
         * TODO: 	we should probably check that no-one tried to declare an 
         * 			outcome variable of this name. As it is, this code will 
         * 			over-write it if such a situation occurs.
         */
        HashMap completionStatus = new HashMap();
        completionStatus.put("identifier", "completionStatus");
        Vector v = new Vector();
        v.add("not_attempted");
        completionStatus.put("completionStatus", v);
        completionStatus.put("cardinality", "single");
        completionStatus.put("baseType", "identifier");
        outVars.put("completionStatus", completionStatus);
        
        return outVars;
    }
    
    private HashMap processTemplateVars(Element e)
    {
        HashMap templVars = new HashMap();
        System.out.println("Root Element is : "+e.getName());
        List list1 = e.getChildren() ;
        Iterator iterator1 = list1.iterator();
        System.out.println("Number of children in ROOT is : "+list1.size());
        while (iterator1.hasNext()) 
         {
              Element child = (Element)iterator1.next();                  
              if(child.getName().equals("templateDeclaration")) 
              {
                  String templDeclName;
                  HashMap templDecl = new HashMap();
                  templDeclName =child.getAttributeValue("identifier");
                  System.out.println("***Value of identifier :"+templDeclName);
                  //Add templateDeclaration attributes to HashMap
                  templDecl.put("identifier",child.getAttributeValue("identifier"));
                  templDecl.put("cardinality",child.getAttributeValue("cardinality"));
                  templDecl.put("baseType",child.getAttributeValue("baseType"));
                  templDecl.put("mathVariable",child.getAttributeValue("mathVariable"));
                  templDecl.put("paramVariable",child.getAttributeValue("paramVariable"));
                  
                  //Get the default Values of outcomeDeclaration tag
                  List list2 = child.getChildren();
                  Iterator iterator2 = list2.iterator();
                  System.out.println("Number of children for"+ child.getName()+" is : "+list2.size());
                  while(iterator2.hasNext()) 
                  {
                     Element child1 = (Element) iterator2.next();
                     if(child1.getName().equals("defaultValue")) //defaultValue can occur only once
                     {
                         templDecl.put("defaultValueInterpretation",child1.getAttributeValue("interpretation"));                                                           
                         //Get all th values for defaultValue
                         Vector res = getDefaultValues(child1);
                         templDecl.put("defaultValue",res);
                     }
                  }
                  if(list2.size()==0) 
                  {
                      templDecl.put("defaultValueInterpretation",null);
                      templDecl.put("defaultValue",null);
                  }
                  
                  templDecl.put(child.getAttributeValue("identifier"),templDecl.get("defaultValue"));
                  
                  //add the responseDeclaration to response variables
                  templVars.put(templDeclName,templDecl);   
              }
         }
        
        return templVars;
    }
    
    private Vector getDefaultValues(Element defaultValue) 
    {
       Vector<Object> values = new Vector<Object>();
       Iterator iterator3 = defaultValue.getChildren().iterator();
       
       while(iterator3.hasNext()) 
       {
            Element child = (Element) iterator3.next();
            String value = child.getTextNormalize();
            values.add(value);
            //HashMap attrValues = new HashMap();
            
            
            /*if(child.getName().equals("value")) //can have multiple values
            {
              attrValues.put("fieldIdentifier",child.getAttributeValue("fieldIdentifier"));
              attrValues.put("baseType",child.getAttributeValue("baseType"));
              values.put(child.getText(),attrValues);              
            }*/            
       } 
       return values;
     }    
     private ArrayList getAreaMapEntryValues(Element areaMapping) 
     {
         ArrayList areaMapEntryValues = new ArrayList();
         List list1 = areaMapping.getChildren();
         Iterator iterator1 = list1.iterator(); 
         while(iterator1.hasNext()) 
         {
             Element child = (Element) iterator1.next();
             if(child.getName().equals("areaMapEntry")) 
             {
                HashMap shapeAttributes = new HashMap();
                String shape =child.getAttributeValue("shape");
                String coords =child.getAttributeValue("coords");
                String mappedValue =child.getAttributeValue("mappedValue");
                shapeAttributes.put("shape",shape);
                shapeAttributes.put("coords",coords);
                shapeAttributes.put("mappedValue",mappedValue); 
                areaMapEntryValues.add(shapeAttributes);
             }
         }
         return areaMapEntryValues;
     }
     
     
     private HashMap getMapEntryValues(Element mapping) 
     {
         HashMap mapEntryValues = new HashMap();
         List list1 = mapping.getChildren();
         Iterator iterator1 = list1.iterator(); 
         while(iterator1.hasNext()) 
         {
              Element child = (Element) iterator1.next();
              if(child.getName().equals("mapEntry")) 
              {
                  String mapKey = child.getAttributeValue("mapKey");
                  String mappedValue = child.getAttributeValue("mappedValue");
                  mapEntryValues.put(mapKey,mappedValue);
              }
         }
         return mapEntryValues;
     }
    
    private Vector  getCorrectResponseValues(Element correctResponse) 
    {
       Vector correctValues = new Vector();
       List list1 = correctResponse.getChildren();
       Iterator iterator1 = list1.iterator(); 
       while(iterator1.hasNext()) 
       {
           Element child = (Element) iterator1.next();
           if(child.getName().equals("value")) 
           {
               correctValues.add(child.getText());
           }        
       }
       return correctValues;
    }
     
}
    

    

