/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing;

import java.awt.Point;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.xml.soap.AttachmentPart;
import javax.xml.soap.SOAPException;

import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;

import r2q2.processing.exception.ConversionException;
import com.gavinwillingham.util.properties.NoSuchPropertyException;
import com.gavinwillingham.util.properties.PropertiesLoader;

import r2q2.NoAttachmentException;
import r2q2.SpecificationViolationException;
import r2q2.processing.expression.Expression;
import r2q2.processing.expression.ExpressionFactory;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.variable.AreaMapEntry;
import r2q2.variable.AreaMapping;
import r2q2.variable.CorrectResponse;
import r2q2.variable.DirectedPair;
import r2q2.util.streams.StreamCopier;
import r2q2.util.xml.DocumentBuilder;
import r2q2.variable.ItemVariable;
import r2q2.variable.MapEntry;
import r2q2.variable.Mapping;
import r2q2.variable.OutcomeDeclaraton;
import r2q2.variable.Pair;
import r2q2.variable.ResponseDeclaration;
import r2q2.variable.TemplateDeclaration;
import r2q2.variable.ItemVariable.BaseType;
import r2q2.variable.ItemVariable.Cardinality;
import r2q2.variable.ItemVariable.Shape;

public class ProcessingEngine {
	
	public static final Namespace ns = Namespace.getNamespace("http://www.imsglobal.org/xsd/imsqti_v2p0");
	
	public HashMap[] processTemplate(HashMap[] vars) throws NoAttachmentException, ProcessingException, SpecificationViolationException
	{
		try{
			
		
			Element root = getRootNode(MessageContext.getCurrentContext().getRequestMessage());
			
			Element templateProcessing = root.getChild("templateProcessing", ns);
			// BUG II (Gavin) Element templateProcessing = root.getChild("responseProcessing", ns);
			if(templateProcessing == null) //no template
				return vars;
	        
			//convert HashMap's to Object structure
			HashMap<String,ItemVariable> objectVars = null;
			try{
				objectVars = populateObjectMaps(vars);
			}catch(ConversionException ce)
			{
				throw new ProcessingException(ce);
			}
			HashMap<String,ItemVariable> responses = null;
			try{
				Iterator it = templateProcessing.getChildren().iterator();
				while(it.hasNext())
					responses = processTemplateElement((Element)it.next(), objectVars);
			}catch(InvalidArgumentsException iae)
			{
				iae.printStackTrace();
				throw new ProcessingException(iae);
			}
			try{
				return rePopulateHashArrays(responses);
			}catch(ConversionException ce)
			{
				throw new ProcessingException("Error: could not repopulate portable legacy objects correctly, aborting",ce);
			}
			
		}catch(Throwable t)
		{
			t.printStackTrace();
			return null;
		}
	}
	
	public HashMap[] processResponse(HashMap[] vars) throws NoAttachmentException, ProcessingException, SpecificationViolationException
	{
		try{
		
			System.out.println("--in--");
			Element root = getRootNode(MessageContext.getCurrentContext().getRequestMessage());
			
			Element responseProcessing = root.getChild("responseProcessing", ns);
			if(responseProcessing == null)
				throw new SpecificationViolationException("Couldn't locate responseProcessing node!");
			
			if(responseProcessing.getChildren().size() == 0){
				byte[] xml = null;
				String templateLocation = responseProcessing.getAttributeValue("templateLocation");
				if(templateLocation != null){
					try {
						URL url = new URL(templateLocation);
						xml = StreamCopier.copyToByteArray(url.openStream());
					} catch (MalformedURLException e) {
						throw new SpecificationViolationException("Invalid response processing template location URL", e);
					} catch (IOException e) {
						throw new SpecificationViolationException("Error reading from remote response processing template", e);
					}
				}
				else{
					String template = responseProcessing.getAttributeValue("template");
					if(template != null){
						String filename = null;
						try{
							if(template.equals("http://www.imsglobal.org/question/qti_v2p0/rptemplates/match_correct")){
								filename = PropertiesLoader.getProperty("r2q2.processing.responsetemplates.matchCorrect");
							}
							else if(template.equals("http://www.imsglobal.org/question/qti_v2p0/rptemplates/map_response")){
								filename = PropertiesLoader.getProperty("r2q2.processing.responsetemplates.mapResponse");					
							}
							else if(template.equals("http://www.imsglobal.org/question/qti_v2p0/rptemplates/map_response_point")){
								filename = PropertiesLoader.getProperty("r2q2.processing.responsetemplates.mapResponsePoint");
							}
							else{
								throw new ProcessingException("Unsupported response processing template");
							}
						} catch(IOException e){
							throw new ProcessingException("Couldn't read properties file", e);
						} catch (NoSuchPropertyException e) {
							throw new ProcessingException("Couldn't find template location property", e);
						}
						
						try {
							xml = StreamCopier.copyToByteArray(
									new FileInputStream(
										new File(filename)));
						} catch (FileNotFoundException e) {
							e.printStackTrace();
							throw new ProcessingException("Couldn't find template", e);
						} catch (IOException e) {
							e.printStackTrace();
							throw new ProcessingException("Couldn't read template", e);
						}
					}
					else
						throw new SpecificationViolationException("No response processing rules!");
				}
				
				//get the root element from the template
				try{
					responseProcessing = DocumentBuilder.getRootElement(xml);
					
					//test it's the right node
					if(!responseProcessing.getName().equals("responseProcessing"))
						throw new SpecificationViolationException("Response template not valid!");
					
					if(responseProcessing.getChildren().size() < 1)
						throw new ProcessingException("No content in template - either a spec violation or an unsupported nested template");
					
				} catch (JDOMException e) {
					throw new SpecificationViolationException("Error building DOM from response processing template", e);
				} catch (NoSuchPropertyException e) {
					throw new ProcessingException(e);
				} catch (IOException e) {
					throw new SpecificationViolationException("Error reading from response processing template", e);
				}
			}
			
			//			convert HashMap's to Object structure
			HashMap<String,ItemVariable> objectVars = null;
			try{
				objectVars = populateObjectMaps(vars);
			}catch(ConversionException ce)
			{
				throw new ProcessingException(ce);
			}
			HashMap<String,ItemVariable> responses = null;
			try{
				Iterator it = responseProcessing.getChildren().iterator();
				while(it.hasNext())
					responses = processResponseElement((Element)it.next(), objectVars);
			}catch(InvalidArgumentsException iae)
			{
				iae.printStackTrace();
				throw new ProcessingException(iae);
			}
			try{
				System.out.println("--out[good]--");
				return rePopulateHashArrays(responses);
			}catch(ConversionException ce)
			{
				throw new ProcessingException("Error: could not repopulate portable legacy objects correctly, aborting",ce);
			}
			
		}catch(Throwable t)
		{
			t.printStackTrace();
			return null;
		}
	}
	
	private Element getRootNode(Message _request) throws NoAttachmentException, ProcessingException{
		//Test for attachments
        if(_request.countAttachments() == 0){			//there are noattachments
        	throw new NoAttachmentException("No supplied XML");
        }
        Iterator attachmentIter = _request.getAttachments();
        AttachmentPart ap = (AttachmentPart)attachmentIter.next();
        
        //extract the attachment, get the root node
        Element root_ = null;
        byte[] xml;
        try {
			xml = StreamCopier.copyToByteArray(ap.getDataHandler().getInputStream());
		} catch (IOException e) {
			throw new ProcessingException("Couldn't process attachment", e);
		} catch (SOAPException e) {
			// TODO Auto-generated catch block
			throw new ProcessingException("Couldn't process attachment", e);
		}		
		
		try {
			return DocumentBuilder.getRootElement(xml);
		} catch (JDOMException e) {
			e.printStackTrace();
			throw new ProcessingException("Couldn't build document", e);
		} catch (IOException e) {
			e.printStackTrace();
			throw new ProcessingException("Couldn't build document", e);
		} catch (NoSuchPropertyException e) {
			e.printStackTrace();
			throw new ProcessingException("Couldn't build document", e);
		}
	}

	private HashMap<String,ItemVariable> processResponseElement(Element _e,HashMap<String,ItemVariable> vars)
												throws InvalidArgumentsException
	{
		String element = _e.getName();
		if(element.equals("exitResponse"))
		{
			//simply return immediately from responseProcessing
			return vars;
		}
		else if(element.equals("responseCondition"))
		{
			//first element is always an if
			Iterator components = _e.getChildren().iterator();
			while(components.hasNext())
			{
				Element ifChild = (Element)components.next();
				
				//attempt to process the 'if'
				Iterator ifChildItr = ifChild.getChildren().iterator();
				//get the first element (evaluation expression)
				if(ifChild.getName().equals("responseElse"))
				{
					while(ifChildItr.hasNext())
						processResponseElement((Element)ifChildItr.next(), vars);
					return vars;
				}else
				{
					Expression exp = ExpressionFactory.getInstance().makeExpression((Element)ifChildItr.next());
					ItemVariable eval = exp.eval(vars);
					if(eval == null)
						continue;
					System.out.println("evaluation variable is : "+eval.value);
					if(eval.card != Cardinality.single || eval.varType != BaseType.bool)
						throw new InvalidArgumentsException("Error: responseIf can only be passed an expression that evaluates to a single cardinality boolean value, aborting");
					if(((Boolean)eval.value).booleanValue())
					{
						//evaluate all further child nodes after the expression and return
						while(ifChildItr.hasNext())
							processResponseElement((Element)ifChildItr.next(), vars);
						return vars;
					}
				}
			}
		}
		else if(element.equals("setOutcomeValue"))
		{
			String ident = _e.getAttribute("identifier").getValue();
			ItemVariable setVar = vars.get(ident);
			//build the expression from the child node
			
			//check that there is only one child
			int size = _e.getChildren().size();
			if( size < 1 || size > 1 )
				throw new InvalidArgumentsException("Error: setOutComeValue can only be called with a single contained expression, aborting");
				
			Expression exp = ExpressionFactory.getInstance().makeExpression((Element)_e.getChildren().iterator().next());
			ItemVariable iv = exp.eval(vars);
			
			if(iv.card == setVar.card && iv.varType == setVar.varType)
			{
				setVar.value = iv.value;
				//TODO may need some more checking here in the form of an elseif to truncate to a plausible value
			}else if(iv.card == setVar.card && (iv.varType == BaseType.afloat || iv.varType == BaseType.integer) &&
											    (setVar.varType == BaseType.afloat || setVar.varType == BaseType.integer))
			{
				if(setVar.varType == BaseType.afloat)
				{
					setVar.value = new Double(((Integer)iv.value).doubleValue());
				}else
				{
					setVar.value = new Integer(((Double)iv.value).intValue());
				}
			}
			else
			{
				System.out.println("iv card : "+iv.card+" setVar card : "+setVar.card+"iv type : "+iv.varType+" setVar type : "+setVar.varType);
				throw new InvalidArgumentsException("Error: assignment of outcome variables must be of same cardinality and baseType as the variable definition, aborting");
			}
			
		}
		else
		{
			System.out.println("element is : "+element);
			throw new InvalidArgumentsException("Error: unacceptable responseRule provided to response processing, aborting");
		}
		return vars;
	}
	
	private HashMap<String, ItemVariable> processTemplateElement(Element _e,HashMap<String, ItemVariable> vars)
											throws InvalidArgumentsException {
		String element = _e.getName();
		System.out.println("element is : "+element);
		if(element.equals("exitTemplate"))
		{
			//simply return immediately from responseProcessing
			return vars;
		}
		else if(element.equals("templateCondition"))
		{
			//first element is always an if
			Iterator components = _e.getChildren().iterator();
			int i = 1;
			while(components.hasNext())
			{
				Element ifChild = (Element)components.next();
				
				//attempt to process the 'if'
				Iterator ifChildItr = ifChild.getChildren().iterator();
				//get the first element (evaluation expression)
				if(ifChild.getName().equals("templateElse"))
				{
					System.out.println("--running and returning template else clause--");
					while(ifChildItr.hasNext())
						processTemplateElement((Element)ifChildItr.next(), vars);
					return vars;
				}else
				{
					Expression exp = ExpressionFactory.getInstance().makeExpression((Element)ifChildItr.next());
					ItemVariable eval = exp.eval(vars);
					if(eval.card != Cardinality.single || eval.varType != BaseType.bool)
						throw new InvalidArgumentsException("Error: responseIf can only be passed an expression that evaluates to a single cardinality boolean value, aborting");
					if(((Boolean)eval.value).booleanValue())
					{
						//evaluate all further child nodes after the expression and return
						System.out.println("--running and returning if / elseif clause ["+i+"] in sequence--");
						while(ifChildItr.hasNext())
							processTemplateElement((Element)ifChildItr.next(), vars);
						return vars;
					}
				}
				i++;
			}
		}
		else if(element.equals("setTemplateValue"))
		{
			String ident = _e.getAttribute("identifier").getValue();
			ItemVariable setVar = vars.get(ident);
			//build the expression from the child node
			
			//check that there is only one child
			int size = _e.getChildren().size();
			if( size < 1 || size > 1 )
				throw new InvalidArgumentsException("Error: setTemplateValue can only be called with a single contained expression, aborting");
				
			Expression exp = ExpressionFactory.getInstance().makeExpression((Element)_e.getChildren().iterator().next());
			ItemVariable iv = exp.eval(vars);
			
			if(iv.card == setVar.card && iv.varType == setVar.varType)
			{
				System.out.println("[TEMPLATE VALUE SETTING] setting template value to "+iv.value);
				setVar.value = iv.value;
				//TODO may need some more checking here in the form of an elseif to truncate to a plausible value
			}
			else if(iv.card == setVar.card && (iv.varType == BaseType.afloat || iv.varType == BaseType.integer) &&
				    (setVar.varType == BaseType.afloat || setVar.varType == BaseType.integer))
			{
				if(setVar.varType == BaseType.afloat)
				{
					setVar.value = new Double(((Integer)iv.value).doubleValue());
				}else
				{
					setVar.value = new Integer(((Double)iv.value).intValue());
				}
			}else
				throw new InvalidArgumentsException("Error: assignment of template variables must be of same cardinality and baseType as the variable definition, aborting");
			
		}
		else if(element.equals("setCorrectResponse"))
		{
			System.out.println("[SETCORRECTRESPONSE] called");
			String ident = _e.getAttribute("identifier").getValue();
			ItemVariable setVar = vars.get(ident);
			if(setVar instanceof ResponseDeclaration)
			{
				ResponseDeclaration rd = (ResponseDeclaration)setVar;
				int size = _e.getChildren().size();
				if( size < 1 || size > 1 )
					throw new InvalidArgumentsException("Error: setCorrectResponse can only be called with a single contained expression, aborting");
					
				Expression exp = ExpressionFactory.getInstance().makeExpression((Element)_e.getChildren().iterator().next());
				ItemVariable iv = exp.eval(vars);
				System.out.println("[SETCORRECTRESPONSE] about to check types");
				if(iv.card == setVar.card && iv.varType == setVar.varType)
				{
					System.out.println("[SETCORRECTRESPONSE] looking at correct response : "+rd.correctResponse);
					if(rd.correctResponse == null)
					{
						rd.correctResponse = new CorrectResponse();
						rd.correctResponse.value = iv.value;
					}
				}
				else if(iv.card == setVar.card && (iv.varType == BaseType.afloat || iv.varType == BaseType.integer) &&
					    (setVar.varType == BaseType.afloat || setVar.varType == BaseType.integer))
				{
					if(setVar.varType == BaseType.afloat)
					{
						if(rd.correctResponse == null)
						{
							rd.correctResponse = new CorrectResponse();
							rd.correctResponse.value = new Double(((Integer)iv.value).doubleValue());
						}
					}else
					{
						if(rd.correctResponse == null)
						{
							rd.correctResponse = new CorrectResponse();
							rd.correctResponse.value = new Integer(((Double)iv.value).intValue());
						}
					}
				}
				else
				{
					System.out.println("iv.card : "+iv.card+" serVar.card : "+setVar.card+ " iv.varType : "+iv.varType+" setVar.varType : "+setVar.varType);
				}
			}else
				throw new InvalidArgumentsException("Error: can only set the correct response on a responseVariable, aborting");
		}
		else if(element.equals("setDefaultValue"))
		{
			String ident = _e.getAttribute("identifier").getValue();
			ItemVariable setVar = vars.get(ident);
			if(setVar instanceof ResponseDeclaration || setVar instanceof OutcomeDeclaraton)
			{
				int size = _e.getChildren().size();
				if( size < 1 || size > 1 )
					throw new InvalidArgumentsException("Error: setCorrectResponse can only be called with a single contained expression, aborting");
					
				Expression exp = ExpressionFactory.getInstance().makeExpression((Element)_e.getChildren().iterator().next());
				ItemVariable iv = exp.eval(vars);
				
				if(iv.card == setVar.card && iv.varType == setVar.varType)
				{
					setVar.defaultValue = iv.value;
				}
			}else
				throw new InvalidArgumentsException("Error: can only set the correct response on a responseVariable, aborting");
		}
		else
			throw new InvalidArgumentsException("Error: unacceptable responseRule provided to response processing, aborting");
		
		return vars;
	}
	
	private static HashMap<String,ItemVariable> populateObjectMaps(HashMap[] vars) throws ConversionException
	{
		HashMap<String,ItemVariable> ret = new HashMap<String,ItemVariable>();
		
		//first populate the reponseVariable types
		Iterator keys = vars[0].keySet().iterator();
		while(keys.hasNext())
		{
			String identifier = (String)keys.next();
			HashMap retrieved = (HashMap)vars[0].get(identifier);
			
			ResponseDeclaration rd = new ResponseDeclaration();
			Iterator components = retrieved.keySet().iterator();
			while(components.hasNext())
			{
				String fieldKey = (String)components.next();
			
				System.out.println("[RESPONSE] "+identifier+" : "+fieldKey+" : "+retrieved.get(fieldKey));
				
				if(fieldKey.equals("identifier")){}//thow away, we already have it}}
				else if(fieldKey.equals("cardinality")){
					String card = (String)retrieved.get(fieldKey);
					if(card.equals("single"))
						rd.card = Cardinality.single;
					else if(card.equals("multiple"))
						rd.card = Cardinality.multiple;
					else if(card.equals("ordered"))
						rd.card = Cardinality.ordered;
					else if(card.equals("record"))
						rd.card = Cardinality.record;
					else
						throw new ConversionException("Error: unknown cardinality type in conversion, aborting");
				}
				else if(fieldKey.equals("baseType"))
				{
					String baseType = (String)retrieved.get(fieldKey);
					if(baseType.equals("float"))
						rd.varType = BaseType.afloat;
					else if(baseType.equals("integer"))
						rd.varType = BaseType.integer;
					else if(baseType.equals("identifier"))
						rd.varType = BaseType.identifier;
					else if(baseType.equals("boolean"))
						rd.varType = BaseType.bool;
					else if(baseType.equals("directedPair"))
						rd.varType = BaseType.directedPair;
					else if(baseType.equals("duration"))
						rd.varType = BaseType.duration;
					else if(baseType.equals("point"))
						rd.varType = BaseType.point;
					else if(baseType.equals("pair"))
						rd.varType = BaseType.pair;
					else if(baseType.equals("string"))
						rd.varType = BaseType.string;
					else if(baseType.equals("file"))
						rd.varType = BaseType.file;
					else if(baseType.equals("uri"))
						rd.varType = BaseType.uri;
					else
						throw new ConversionException("Error: unknown cardinality type in conversion, aborting");
				}
				else if(fieldKey.equals("candidateValues"))
				{
					Object retObj = retrieved.get(fieldKey);
					rd.value = retObj;
				}
				else if(fieldKey.equals("defaultValue"))
				{
					Object defaultValue = retrieved.get(fieldKey);
					rd.defaultValue = defaultValue;
				}
				else if(fieldKey.equals("defaultValueInterpretation"))
				{
					String defaultValueInterpretation = (String)retrieved.get(fieldKey);
					rd.defaultValueInterpretation = defaultValueInterpretation;
				}else if(fieldKey.equals("correctResponse"))
				{
					Object correctResp = retrieved.get(fieldKey);
					rd.correctResponse = new CorrectResponse();
					rd.correctResponse.value = correctResp;
				}else if(fieldKey.equals("mappingDefaultValue"))
				{
					if (rd.mapping == null)
						rd.mapping = new Mapping();
					if(retrieved.get(fieldKey) == null)
						rd.mapping.defaultValue = null;
					else
						rd.mapping.defaultValue = Double.parseDouble((String)retrieved.get(fieldKey));
				}else if(fieldKey.equals("mappingLowerBound"))
				{
					if (rd.mapping == null)
						rd.mapping = new Mapping();
					if(retrieved.get(fieldKey) == null)
						rd.mapping.lowerBound = null;
					else
						rd.mapping.lowerBound = Double.parseDouble((String)retrieved.get(fieldKey));
				}else if(fieldKey.equals("mappingUpperBound"))
				{
					if (rd.mapping == null)
						rd.mapping = new Mapping();
					if(retrieved.get(fieldKey) == null)
						rd.mapping.upperBound = null;
					else
						rd.mapping.upperBound = Double.parseDouble((String)retrieved.get(fieldKey));
				}else if(fieldKey.equals("mapEntry"))
				{
					if (rd.mapping == null)
						rd.mapping = new Mapping();
					
					if(rd.mapping.mapEntries == null)
						rd.mapping.mapEntries = new Vector<MapEntry>();
					
					if(retrieved.get(fieldKey) != null)
					{
						HashMap mapHash = (HashMap)retrieved.get(fieldKey);
						Iterator mapKeys = mapHash.keySet().iterator();
						while(mapKeys.hasNext())
						{
							String key = (String)mapKeys.next();
							Double value = Double.parseDouble((String)mapHash.get(key));
							rd.mapping.mapEntries.add(new MapEntry(key,value));
						}
					}
				}else if(fieldKey.equals("areaMappingDefaultValue"))
				{
					if (rd.areaMapping == null)
						rd.areaMapping = new AreaMapping();
					if(retrieved.get(fieldKey) == null)
						rd.areaMapping.defaultValue = null;
					else
						rd.areaMapping.defaultValue = Double.parseDouble((String)retrieved.get(fieldKey));
				}else if(fieldKey.equals("areaMappingLowerBound"))
				{
					if (rd.areaMapping == null)
						rd.areaMapping = new AreaMapping();
					if(retrieved.get(fieldKey) == null)
						rd.areaMapping.lowerBound = null;
					else
						rd.areaMapping.lowerBound = Double.parseDouble((String)retrieved.get(fieldKey));
				}else if(fieldKey.equals("areaMappingUpperBound"))
				{
					if (rd.areaMapping == null)
						rd.areaMapping = new AreaMapping();
					if(retrieved.get(fieldKey) == null)
						rd.areaMapping.upperBound = null;
					else
						rd.areaMapping.upperBound = Double.parseDouble((String)retrieved.get(fieldKey));
				}else if(fieldKey.equals("areaMapEntry"))
				{
					if (rd.areaMapping == null)
						rd.areaMapping = new AreaMapping();
					
					if(rd.areaMapping.areaMapEntries == null)
						rd.areaMapping.areaMapEntries = new Vector<AreaMapEntry>();
					
					if(retrieved.get(fieldKey) != null)
					{
						//TIM: ArrayList areaMapList = (ArrayList)retrieved.get(fieldKey);
						//TIM: Iterator areaMaps = areaMapList.iterator();
						
						Object[] areaMapList = (Object[])retrieved.get(fieldKey);
						
						//do for all elements
						//TIM: while(areaMaps.hasNext())
						for(int i=0; i<areaMapList.length; i++)
						{
							HashMap<String,String> element = (HashMap<String,String>)areaMapList[i];
							AreaMapEntry ame = new AreaMapEntry();
							
							String shapeType = element.get("shape");
							if(shapeType == null)
								throw new ConversionException("Error: no shape provided in the legacy format, aborting");
							if(shapeType.equals("circle"))
								ame.shape = Shape.circle;
							else if(shapeType.equals("rect"))
								ame.shape = Shape.rect;
							else if(shapeType.equals("poly"))
								ame.shape = Shape.poly;
							else if(shapeType.equals("ellipse"))
								ame.shape = Shape.ellipse;
							else
								throw new ConversionException("Error: default or other shapes not acceptable in this implementation, aborting");
							String coords = element.get("coords");
							if(coords == null)
								throw new ConversionException("Error: no coords provided in legacy format, aborting");
							StringTokenizer st = new StringTokenizer(coords,",");
							ame.coords = new Double[st.countTokens()];
							for(int j=0; j<ame.coords.length; j++)
								ame.coords[j] = new Double(st.nextToken());
							
							if(element.get("mappedValue") == null)
								throw new ConversionException("Error: a mapping must contain a value in the legacy format, aborting");
							ame.mappedValue = new Double(element.get("mappedValue"));
							rd.areaMapping.areaMapEntries.add(ame);
						}
					}
				}
			}
			
			//unpack Objects according to baseType
			
			if(rd.correctResponse != null)
				rd.correctResponse.value = unpack(rd.correctResponse.value, rd.varType);
			if(rd.mapping != null)
			{
				System.out.println("unpacking the mapping field");
				rd.mapping.mapEntries = unpackMapping(rd.mapping.mapEntries,rd.varType);
			}
			rd.defaultValue = unpack(rd.defaultValue, rd.varType);
			System.out.println("result of unpacked (default) value is : "+rd.defaultValue);
			rd.value = unpack(rd.value, rd.varType);
			System.out.println("result of unpacked value is : "+rd.value);
			ret.put(identifier,rd);
		}
		
		//then populate the outcomeVariable types
		keys = vars[1].keySet().iterator();
		while(keys.hasNext())
		{
			String identifier = (String)keys.next();
			HashMap retrieved = (HashMap)vars[1].get(identifier);
			
			OutcomeDeclaraton od = new OutcomeDeclaraton();
			Iterator components = retrieved.keySet().iterator();
			while(components.hasNext())
			{
				String fieldKey = (String)components.next();
				
				System.out.println("[OUTCOME] "+identifier+" : "+fieldKey+" : "+retrieved.get(fieldKey));
				
				if(fieldKey.equals(identifier))
				{
					od.value = retrieved.get(identifier);
				}
				else if(fieldKey.equals("identifier")){
					Object retObj = retrieved.get(fieldKey);
					if(retObj != null)
					{
						od.identifier = (String)retObj;
					}
				}
				else if(fieldKey.equals("cardinality")){
					String card = (String)retrieved.get(fieldKey);
					if(card.equals("single"))
						od.card = Cardinality.single;
					else if(card.equals("multiple"))
						od.card = Cardinality.multiple;
					else if(card.equals("ordered"))
						od.card = Cardinality.ordered;
					else if(card.equals("record"))
						od.card = Cardinality.record;
					else
						throw new ConversionException("Error: unknown cardinality type in conversion, aborting");
				}
				else if(fieldKey.equals("baseType"))
				{
					System.out.println("has found a baseType");
					String baseType = (String)retrieved.get(fieldKey);
					if(baseType.equals("float"))
						od.varType = BaseType.afloat;
					else if(baseType.equals("integer"))
						od.varType = BaseType.integer;
					else if(baseType.equals("identifier"))
						od.varType = BaseType.identifier;
					else if(baseType.equals("boolean"))
						od.varType = BaseType.bool;
					else if(baseType.equals("directedPair"))
						od.varType = BaseType.directedPair;
					else if(baseType.equals("duration"))
						od.varType = BaseType.duration;
					else if(baseType.equals("point"))
						od.varType = BaseType.point;
					else if(baseType.equals("pair"))
						od.varType = BaseType.pair;
					else if(baseType.equals("string"))
						od.varType = BaseType.string;
					else if(baseType.equals("file"))
						od.varType = BaseType.file;
					else if(baseType.equals("uri"))
						od.varType = BaseType.uri;
					else
						throw new ConversionException("Error: unknown cardinality type in conversion, aborting");
				}
				else if(fieldKey.equals("defaultValue"))
				{
					Object defaultValue = retrieved.get(fieldKey);
					od.defaultValue = defaultValue;
				}
				else if(fieldKey.equals("defaultValueInterpretation"))
				{
					String defaultValueInterpretation = (String)retrieved.get(fieldKey);
					od.defaultValueInterpretation = defaultValueInterpretation;
				}
				else if(fieldKey.equals("interpretation"))
				{
					String interpretation = (String)retrieved.get(fieldKey);
					od.interpretation = interpretation;
				}
				else if(fieldKey.equals("longInterpretation"))
				{
					String longInterpretation = (String)retrieved.get(fieldKey);
					od.longInterpretation = longInterpretation;
				}
				else if(fieldKey.equals("normalMaximum"))
				{
					if(retrieved.get(fieldKey) == null)
						od.normalMaximum = null;
					else
						od.normalMaximum = new Double((String)retrieved.get(fieldKey));
				}
			}
			od.defaultValue = unpack(od.defaultValue, od.varType);
			od.value = unpack(od.value, od.varType);
			ret.put(identifier,od);
		}
		
		//finally poplulate the templateVariable types
		keys = vars[2].keySet().iterator();
		while(keys.hasNext())
		{
			
			String identifier = (String)keys.next();
			HashMap retrieved = (HashMap)vars[2].get(identifier);
			
			TemplateDeclaration td = new TemplateDeclaration();
			Iterator components = retrieved.keySet().iterator();
			while(components.hasNext())
			{
				String fieldKey = (String)components.next();
				System.out.println("[TEMPLATE] "+identifier+" : "+fieldKey+" : "+retrieved.get(fieldKey));
				if(fieldKey.equals(identifier))
				{
					td.value = retrieved.get(identifier);
				}
				else if(fieldKey.equals("identifier")){
					Object retObj = retrieved.get(fieldKey);
					if(retObj != null)
					{
						td.identifier = (String)retObj;
					}
				}
				else if(fieldKey.equals("cardinality")){
					String card = (String)retrieved.get(fieldKey);
					if(card.equals("single"))
						td.card = Cardinality.single;
					else if(card.equals("multiple"))
						td.card = Cardinality.multiple;
					else if(card.equals("ordered"))
						td.card = Cardinality.ordered;
					else if(card.equals("record"))
						td.card = Cardinality.record;
					else
						throw new ConversionException("Error: unknown cardinality type in conversion, aborting");
				}
				else if(fieldKey.equals("baseType"))
				{
					String baseType = (String)retrieved.get(fieldKey);
					if(baseType.equals("float"))
						td.varType = BaseType.afloat;
					else if(baseType.equals("integer"))
						td.varType = BaseType.integer;
					else if(baseType.equals("identifier"))
						td.varType = BaseType.identifier;
					else if(baseType.equals("boolean"))
						td.varType = BaseType.bool;
					else if(baseType.equals("directedPair"))
						td.varType = BaseType.directedPair;
					else if(baseType.equals("duration"))
						td.varType = BaseType.duration;
					else if(baseType.equals("point"))
						td.varType = BaseType.point;
					else if(baseType.equals("pair"))
						td.varType = BaseType.pair;
					else if(baseType.equals("string"))
						td.varType = BaseType.string;
					else if(baseType.equals("file"))
						td.varType = BaseType.file;
					else if(baseType.equals("uri"))
						td.varType = BaseType.uri;
					else
						throw new ConversionException("Error: unknown cardinality type in conversion, aborting");
				}
				else if(fieldKey.equals("defaultValue"))
				{
					Object defaultValue = retrieved.get(fieldKey);
					td.defaultValue = defaultValue;
				}
				else if(fieldKey.equals("defaultValueInterpretation"))
				{
					String defaultValueInterpretation = (String)retrieved.get(fieldKey);
					td.defaultValueInterpretation = defaultValueInterpretation;
				}
				else if(fieldKey.equals("paramVariable"))
				{
					if(retrieved.get(fieldKey) == null)
						td.paramVariable = null;
					else
						td.paramVariable = new Boolean((String)retrieved.get(fieldKey));
				}
				else if(fieldKey.equals("mathVariable"))
				{ 
					if(retrieved.get(fieldKey) == null)
						td.mathVariable = null;
					else
						td.mathVariable = new Boolean((String)retrieved.get(fieldKey));
				}
			}
			td.defaultValue = unpack(td.defaultValue, td.varType);
			td.value = unpack(td.value, td.varType);
			ret.put(identifier,td);
		}
		return ret;
	}
	
	private static HashMap[] rePopulateHashArrays(HashMap<String,ItemVariable> vars) throws ConversionException
	{
		HashMap[] ret = new HashMap[3];
		for(int i=0; i<3; i++)
			ret[i] = new HashMap();
		
		
		Iterator it = vars.keySet().iterator();
		while(it.hasNext())
		{
			String key = (String)it.next();
			Object test = vars.get(key);
			if(test instanceof ResponseDeclaration)
			{
				ret[0].put(key, ((ResponseDeclaration)test).toLegacyFormat());
			}else if(test instanceof OutcomeDeclaraton)
			{
				ret[1].put(key, ((OutcomeDeclaraton)test).toLegacyFormat());
			}else if(test instanceof TemplateDeclaration)
			{
				ret[2].put(key, ((TemplateDeclaration)test).toLegacyFormat());
			}else
				throw new ConversionException("Error: in converting the arrays to legacy format an unknown variable type has appreared, aborting");
		}
		return ret;
	}
	
	private static Vector<MapEntry> unpackMapping(Vector<MapEntry> entries, BaseType type)
	{
		System.out.println("unpacking the [correct] way");
		Vector<MapEntry> rets = new Vector<MapEntry>();
		
		for(int i =0; i < entries.size(); i++)
		{
			String key = (String)entries.elementAt(i).mapKey;
			Double value = entries.elementAt(i).mappedValue;
			if(type == BaseType.integer)
			{
				rets.add(new MapEntry(new Integer(key),value));
			}else if(type == BaseType.afloat)
			{
				rets.add(new MapEntry(new Double(key),value));
			}else if(type == BaseType.bool)
			{
				rets.add(new MapEntry(new Boolean(key),value));
			}else if(type == BaseType.directedPair)
			{
				//tokenize and split
				StringTokenizer st = new StringTokenizer(key);
				rets.add(new MapEntry(new DirectedPair(st.nextToken(),st.nextToken()),value));
			}else if(type == BaseType.pair)
			{
				//tokenize and split
				System.out.println("creating a pair...");
				StringTokenizer st = new StringTokenizer(key);
				rets.add(new MapEntry(new Pair(st.nextToken(),st.nextToken()),value));
			}else if(type == BaseType.point)
			{
				//tokenize and split
				StringTokenizer st = new StringTokenizer(key);
				rets.add(new MapEntry(new Point(Integer.parseInt(st.nextToken()),Integer.parseInt(st.nextToken())),value));
			}else
				rets.add(entries.elementAt(i));
		}
		return rets;
	}
	
	private static Object unpack(Object target, BaseType type)
	{
		Vector<Object> tmp = new Vector<Object>();
		Vector<Object> ret = new Vector<Object>();
		if(target == null)
			return null;
		if(target instanceof Vector)
		{
			System.out.println("Unpacking vector");
			tmp = (Vector)target;
		}else
		{
			System.out.println("Unpacking single value");
			tmp.add(target);
		}
		for(int i=0; i<tmp.size();i++)
		{
			//unpack the elements
			if(type == BaseType.integer)
			{
				ret.add(new Integer((String)tmp.elementAt(i)));
			}else if(type == BaseType.afloat)
			{
				ret.add(new Double((String)tmp.elementAt(i)));
			}else if(type == BaseType.bool)
			{
				ret.add(new Boolean((String)tmp.elementAt(i)));
			}else if(type == BaseType.directedPair)
			{
				//tokenize and split
				StringTokenizer st = new StringTokenizer((String)tmp.elementAt(i));
				ret.add(new DirectedPair(st.nextToken(),st.nextToken()));
			}else if(type == BaseType.pair)
			{
				//tokenize and split
				StringTokenizer st = new StringTokenizer((String)tmp.elementAt(i));
				ret.add(new Pair(st.nextToken(),st.nextToken()));
			}else if(type == BaseType.point)
			{
				//tokenize and split
				StringTokenizer st = new StringTokenizer((String)tmp.elementAt(i));
				ret.add(new Point(Integer.parseInt(st.nextToken()),Integer.parseInt(st.nextToken())));
			}else if(target instanceof Vector){
				System.out.println("is a vector in unpack");
				ret.add(tmp.elementAt(i));
			}
			else
				ret.add(target);
		}	
		
		/*if(ret.size() == 1)
			return ret.elementAt(0);
		else*/
			return ret;
	}
}
