/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression;

import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import org.jdom.Element;

import r2q2.processing.expression.Expression;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.processing.expression.ExpressionFactory;
import r2q2.variable.ItemVariable;
import r2q2.variable.TransitoryVariable;
import r2q2.variable.ItemVariable.BaseType;
import r2q2.variable.ItemVariable.Cardinality;

/**
 * 
 * baseValue operator expression
 * 
 * constraints: 
 *   creates a type of a given baseType and returns the formed object
 */
public class BaseValue implements Expression {
	
	String tagData;
	BaseType baseType;
	
	public BaseValue(Element _e) throws InvalidArgumentsException
	{
		
		tagData = _e.getTextNormalize();
		String baseTypeStr = _e.getAttribute("baseType").getValue();

		if(baseTypeStr.equals("boolean")) baseType = BaseType.bool;
		else if(baseTypeStr.equals("coords")) baseType = BaseType.coords;
		else if(baseTypeStr.equals("duration")) baseType = BaseType.duration;
		else if(baseTypeStr.equals("float")) baseType = BaseType.afloat;
		else if(baseTypeStr.equals("identifier")) baseType = BaseType.identifier;
		else if(baseTypeStr.equals("integer")) baseType = BaseType.integer;
		else if(baseTypeStr.equals("string")) baseType = BaseType.string;
		else if(baseTypeStr.equals("point")) baseType = BaseType.point;
		else if(baseTypeStr.equals("pair")) baseType = BaseType.pair;
		else if(baseTypeStr.equals("directedPair")) baseType = BaseType.directedPair;
		else if(baseTypeStr.equals("uri")) baseType = BaseType.uri;
		else throw new InvalidArgumentsException("Error: "+baseTypeStr+" is not a supported primative dataType within this Implementation," +
				                                  " please refer to documentation and IMS specification, aborting");
		
		
	}
	
	public ItemVariable eval(HashMap vars) throws InvalidArgumentsException
	{
		switch(baseType)
		{
			case bool: return new TransitoryVariable(BaseType.bool,Cardinality.single,new Boolean(tagData));
			case coords: 
				StringTokenizer st = new StringTokenizer(tagData,",");
				Double[] coords = new Double[st.countTokens()];
				try{
					for(int i=0;st.hasMoreTokens();i++)
						coords[i] = new Double(st.nextToken());
				}catch(NumberFormatException nfe)
				{
					throw new InvalidArgumentsException("Error: baseType coords can only be instantiated with comma seperated numeric format data, aborting",nfe);
				}
				return new TransitoryVariable(BaseType.coords,Cardinality.single,coords);
			case duration: 
				throw new InvalidArgumentsException("Error: duration baseType not yet implemented, unsure how to handle, aborting");
			case afloat: 
				try{
					return new TransitoryVariable(BaseType.afloat,Cardinality.single,new Double(tagData));
				}catch(NumberFormatException nfe)
				{
					throw new InvalidArgumentsException("Error: baseType 'float' can only be instantiated with a numericly formated string, aborting");
				}
			case identifier: 
				return new TransitoryVariable(BaseType.identifier,Cardinality.single,tagData);
			case integer: 
				return new TransitoryVariable(BaseType.integer,Cardinality.single,new Integer(tagData));
			case string: 
				return new TransitoryVariable(BaseType.string,Cardinality.single,tagData);
			case pair:
				//split the pair
				StringTokenizer st2 = new StringTokenizer(tagData);
				if(st2.countTokens() != 2)
					throw new InvalidArgumentsException("Error: pair data must contain two space-seperated values, aborting");
				Vector<String> passVec = new Vector<String>();
				passVec.add(st2.nextToken());
				passVec.add(st2.nextToken());
				return new TransitoryVariable(BaseType.identifier,Cardinality.single,passVec);
			case directedPair:
				StringTokenizer st3 = new StringTokenizer(tagData);
				if(st3.countTokens() != 2)
					throw new InvalidArgumentsException("Error: pair data must contain two space-seperated values, aborting");
				Vector<String> passVec2 = new Vector<String>();
				passVec2.add(st3.nextToken());
				passVec2.add(st3.nextToken());
				return new TransitoryVariable(BaseType.identifier,Cardinality.single,passVec2);
			case point:
				StringTokenizer st4 = new StringTokenizer(tagData);
				if(st4.countTokens() != 2)
					throw new InvalidArgumentsException("Error: pair data must contain two space-seperated values, aborting");
				Vector<String> passVec3 = new Vector<String>();
				try{
					passVec3.add(st4.nextToken());
					passVec3.add(st4.nextToken());
				}catch(NumberFormatException nfe)
				{
					throw new InvalidArgumentsException("Error: data pairs for a 'point' must be of type 'integer', aborting");
				}
				return new TransitoryVariable(BaseType.identifier,Cardinality.single,passVec3);
			case uri: 
				return new TransitoryVariable(BaseType.uri,Cardinality.single,tagData);
			default:
					throw new InvalidArgumentsException("Error: trying to evaluate an unsupported base type object, aborting");
		}
	}

}