/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;

import org.jdom.Element;

import r2q2.processing.expression.operators.IsNull;
import r2q2.processing.expression.operators.Multiple;
import r2q2.processing.expression.operators.Ordered;

public class ExpressionFactory {
	
	public static ExpressionFactory instance = null;
	
	public static ExpressionFactory getInstance()
	{
		
		if (instance == null)
			instance = new ExpressionFactory();
		return instance;
	}
	
	public static HashMap<String,Boolean> generalExpressions = null;
	
	public static HashMap<String,Boolean> getGeneralExpressions()
	{
		if(generalExpressions == null)
		{
			generalExpressions = new HashMap<String,Boolean>();
			generalExpressions.put("baseValue", new Boolean(true));
			generalExpressions.put("correct", new Boolean(true));
			generalExpressions.put("default", new Boolean(true));
			generalExpressions.put("mapResponse", new Boolean(true));
			generalExpressions.put("mapResponsePoint", new Boolean(true));
			generalExpressions.put("randomFloat", new Boolean(true));
			generalExpressions.put("randomInteger", new Boolean(true));
			generalExpressions.put("variable", new Boolean(true));
		}
		return generalExpressions;
	}
	

	public Expression makeExpression(Element _e) throws InvalidArgumentsException
	{
		String elementName = _e.getName();
		char[] tmp = elementName.toCharArray();
		tmp[0] -= 32;
		String loadName = "";
		if(getGeneralExpressions().get(elementName) != null)
			loadName = "r2q2.processing.expression." + new String(tmp);
		else
			loadName = "r2q2.processing.expression.operators." + new String(tmp);
		Class expressionClass = null;
		try
		{
			System.out.println("class name is : "+loadName);
			expressionClass = Class.forName(loadName);
			System.out.println("got 1");
			Class[] paramTypes = new Class[] {Element.class};
			System.out.println("got 2");
			Constructor cons = expressionClass.getConstructor(paramTypes);
			System.out.println("got 3");
			Object[] args = new Element[] {_e};
			System.out.println("got 4");
			Expression exp = (Expression)cons.newInstance(args);
			System.out.println("got 5");
			return exp;
		}catch(ClassNotFoundException cnfe)
		{
			throw new InvalidArgumentsException("Error: no class, and so no method exists for -->"+elementName);
		}catch(IllegalAccessException iae)
		{
			throw new InvalidArgumentsException("Error: method class cannot be accessed from this point -->"+elementName);
		}catch(InstantiationException ie)
		{
			throw new InvalidArgumentsException("Error: method class cannot be instantiated in this manner -->"+elementName);
		}catch(InvocationTargetException ite)
		{
			ite.printStackTrace();
			throw new InvalidArgumentsException("Error: failure occured during instantiation of class constructor for -->"+elementName);
		}catch(NoSuchMethodException nsme)
		{
			throw new InvalidArgumentsException("Error: could not locate the class constructor for -->"+elementName);
		}
		/*if(elementName.equals("multiple")) return new Multiple(_e);
		if(elementName.equals("ordered")) return new Ordered(_e);
		if(elementName.equals("isNull")) return new IsNull(_e);*/
	}
}
