/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.jdom.Element;

import r2q2.processing.expression.Expression;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.variable.ItemVariable;
import r2q2.variable.MapEntry;
import r2q2.variable.ResponseDeclaration;
import r2q2.variable.TransitoryVariable;
import r2q2.variable.ItemVariable.BaseType;
import r2q2.variable.ItemVariable.Cardinality;

/**
 * 
 * 'mapResponse' operator expression
 * 
 * constraints: 
 *   retrieves a variable from the hash store and sums its values by checking the map response for
 *   each combination, summing them accordingly and returning the single float (Double).
 */
public class MapResponse implements Expression {
	
	String identifier;
	
	public MapResponse(Element _e) throws InvalidArgumentsException
	{
		identifier = _e.getAttributeValue("identifier");
	}
	
	public ItemVariable eval(HashMap<String,ItemVariable> vars) throws InvalidArgumentsException
	{
		System.out.println("in map eval");
		ItemVariable retrieve = vars.get(identifier);
		if(retrieve instanceof ResponseDeclaration)
		{
			ResponseDeclaration rd = (ResponseDeclaration)retrieve;
			if(rd.mapping == null)
				throw new InvalidArgumentsException("Error: mapping within responseVariable MUST be declared in order to perform a mapResponse, aborting");
			
			//for each value, test against the map for the score
			if(rd.card == Cardinality.single)
			{
				int i = 0;
				Double tmp = rd.mapping.getValue(rd.value);
				System.out.println("object tmp is : "+tmp);
				if(tmp != null)
				{
					System.out.println("tmp var is NOT null");
					double value =tmp.doubleValue();
					System.out.println("value is : ");
					if(rd.mapping.lowerBound != null)
					{
						if(value < (rd.mapping.lowerBound).doubleValue())
							value = (rd.mapping.lowerBound).doubleValue();
					}
					if(rd.mapping.upperBound != null)
					{
						if(value > (rd.mapping.upperBound).doubleValue())
							value = (rd.mapping.upperBound).doubleValue();
					}
					
					return new TransitoryVariable(BaseType.afloat,Cardinality.single,new Double(value));
				}
				else
					return new TransitoryVariable(BaseType.afloat,Cardinality.single,rd.mapping.defaultValue);
			}else
			{
				System.out.println("cardinality IS multiple");
				Vector<Object> values = (Vector<Object>)rd.value;
				
				if(values ==  null)
					return new TransitoryVariable(BaseType.afloat,Cardinality.single,rd.mapping.defaultValue);
				
				//remove any duplicates from the values array
				for(int i=0; i< values.size(); i++)
				{
					for( int j = i+1; j<values.size(); j++)
					{
						if(values.elementAt(i).equals(values.elementAt(j)))
						{
							values.removeElementAt(j);
							--j;
						}	
					}
				}
				System.out.println("after repeat removal array size is : "+values.size());
				double value = 0;
				//TODO remove this
				Iterator it = rd.mapping.mapEntries.iterator();
				System.out.println("----start of values----");
				while(it.hasNext())
				{
					System.out.println("token is : "+it.next());
				}
				System.out.println("----end of values----");
				
				for(int i=0; i < values.size(); i++)
				{
					int j = 0;
					int size = rd.mapping.mapEntries.size();
					
					System.out.println("rd value is : "+values.elementAt(i));
					Double tmp = rd.mapping.getValue(values.elementAt(i));
					if(tmp != null)
					{
						value +=tmp.doubleValue();
					}
					else
						value += (rd.mapping.defaultValue).doubleValue();
				}
				if(rd.mapping.lowerBound != null)
				{
					if(value < (rd.mapping.lowerBound).doubleValue())
						value = (rd.mapping.lowerBound).doubleValue();
				}
				if(rd.mapping.upperBound != null)
				{
					if(value > (rd.mapping.upperBound).doubleValue())
						value = (rd.mapping.upperBound).doubleValue();
				}
				return new TransitoryVariable(BaseType.afloat,Cardinality.single,new Double(value));
			}
			
		}
		throw new InvalidArgumentsException("Error: variable retrieved from the HashMap is not type responseVariable, aborting");
	}

}