/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression;

import java.awt.Point;
import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.Vector;

import org.jdom.Element;

import r2q2.processing.expression.operators.Inside;
import r2q2.variable.AreaMapEntry;
import r2q2.variable.ItemVariable;
import r2q2.variable.ResponseDeclaration;
import r2q2.variable.TransitoryVariable;
import r2q2.variable.ItemVariable.BaseType;
import r2q2.variable.ItemVariable.Cardinality;
import r2q2.variable.ItemVariable.Shape;

/**
 * 
 * 'mapResponsePoint' operator expression
 * 
 * constraints: 
 *   retrieves a variable from the hash store and sums its values by checking the map response for
 *   each combination, summing them accordingly and returning the single float (Double) response.
 */
public class MapResponsePoint implements Expression {
	
	String identifier;
	
	public MapResponsePoint(Element _e) throws InvalidArgumentsException
	{
		identifier = _e.getAttributeValue("identifier");
	}
	
	public ItemVariable eval(HashMap<String,ItemVariable> vars) throws InvalidArgumentsException
	{
		ItemVariable retrieve = vars.get(identifier);
		if(retrieve instanceof ResponseDeclaration)
		{
			ResponseDeclaration rd = (ResponseDeclaration)retrieve;
			if(rd.areaMapping == null)
				throw new InvalidArgumentsException("Error: areaMapping within responseVariable MUST be declared in order to perform a mapResponsePoint, aborting");
			
			//for each value, test against the map for the score
			Vector<Object> values;
			if(rd.card == Cardinality.single)
			{
				values = new Vector<Object>();
				values.add(rd.value);
			}else if( rd.card == Cardinality.multiple || rd.card == Cardinality.ordered)
			{
				values = (Vector<Object>)rd.value;
			}else
				throw new InvalidArgumentsException("Error: areaMapping can only be called on a response value of single, multiple or ordered cardinality, not record, aborting");
			
			//false until a true intersection found
			boolean[] seen = new boolean[rd.areaMapping.areaMapEntries.size()];
			for(int i=0; i < seen.length; i++)
				seen[i] = false;
			
			double value = 0;
			for(int i=0; i<values.size(); i++)
			{
				System.out.println("Class of element: " + values.elementAt(i).getClass());
				if(values.elementAt(i) instanceof java.awt.Point)
				{
					Point test = (Point)values.elementAt(i);
					//test point into the coords element
					for(int j=0; j < rd.areaMapping.areaMapEntries.size(); j++)
					{
						if(!seen[j])
						{
							AreaMapEntry tmp = rd.areaMapping.areaMapEntries.elementAt(j);
							//format coords into a double[]
							double[] coords = new double[tmp.coords.length];
							for(int k=0; k<coords.length; k++)
								coords[k] = tmp.coords[k].doubleValue();
							boolean inside = false;
							switch((Shape)tmp.shape)
							{
								case circle : inside = Inside.insideCircle(test,coords).booleanValue(); break;
								case rect: inside = Inside.insideRect(test,coords).booleanValue(); break;
								case poly: inside = Inside.insidePolygon(test,coords).booleanValue(); break;
								case ellipse: inside = Inside.insideEllipse(test,coords).booleanValue(); break;
								case defaults: throw new InvalidArgumentsException("Error: this implementation does not provide for a default shape, please use another method, aborting");
							}
							if(inside)
							{
								value += ((Double)tmp.mappedValue).doubleValue();
								seen[j] = true;
							}
						}
					}
				}else
					throw new InvalidArgumentsException("Error: only integer point types are acceptable response types for a mapResponsePoint, aborting");
			}
			
			if(rd.areaMapping.lowerBound != null)
			{
				if(value < (rd.areaMapping.lowerBound).doubleValue())
					value = (rd.areaMapping.lowerBound).doubleValue();
			}
			if(rd.areaMapping.upperBound != null)
			{
				if(value > (rd.areaMapping.upperBound).doubleValue())
					value = (rd.areaMapping.upperBound).doubleValue();
			}
			return new TransitoryVariable(BaseType.afloat,Cardinality.single,new Double(value));
			
		}
		throw new InvalidArgumentsException("Error: variable retrieved from the HashMap is not type responseVariable, aborting");
	}

}