/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression;

import java.util.HashMap;
import java.util.Random;

import org.jdom.Element;

import r2q2.processing.expression.Expression;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.variable.ItemVariable;
import r2q2.variable.TransitoryVariable;
import r2q2.variable.ItemVariable.BaseType;
import r2q2.variable.ItemVariable.Cardinality;

/**
 * 
 * RandomInteger operator expression
 * 
 * constraints: 
 *   generates a random integer within a range and to a given step size
 */
public class RandomInteger implements Expression {
	
	private static Random rGen = null;
	
	private static Random getRandom()
	{
		if (rGen == null)
			rGen = new Random();
		return rGen;
	}
	
	int min = 0;
	int max;
	int step = 1;
	
	
	public RandomInteger(Element _e) throws InvalidArgumentsException
	{
		if(_e.getAttributeValue("min") != null)
			min = Integer.parseInt(_e.getAttributeValue("min"));
		if(_e.getAttributeValue("max") != null)
			max = Integer.parseInt(_e.getAttributeValue("max"));
		if(_e.getAttributeValue("step") != null)
			step = Integer.parseInt(_e.getAttributeValue("step"));
		if(step == 0)
			step = 1;
	}
	
	
	
	public ItemVariable eval(HashMap<String,ItemVariable> vars) throws InvalidArgumentsException
	{
		return new TransitoryVariable(BaseType.integer,Cardinality.single,new Integer((getRandom().nextInt((max-min)/step)*step)+min));
	}

}