/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression.operators;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.jdom.Element;

import r2q2.processing.expression.Expression;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.processing.expression.ExpressionFactory;
import r2q2.variable.ItemVariable;
import r2q2.variable.TransitoryVariable;
import r2q2.variable.ItemVariable.Cardinality;
import r2q2.variable.ItemVariable.BaseType;

/**
 * 
 * And operator expression
 * 
 * constraints: 
 *   1 or more child expressions
 *   takes and returns a base types of boolean only, single cardinality
 *   returns Boolean 'true' if all sub-expressions are true, otherwise false
 *   if sub-expression is NULL, it also returns null
 */
public class And implements Expression {

	private Vector<Expression> subparts = new Vector<Expression>();
	
	public And(Element _e) throws InvalidArgumentsException
	{
		//get from the factory an expression for each child element (Node)
		ExpressionFactory of = ExpressionFactory.getInstance();
		Iterator children = _e.getChildren().iterator();
		int i=0;
		try{
			while(children.hasNext()){
	    		subparts.add(of.makeExpression((Element)children.next()));
	    		i++;
	    	}
    	}catch(InvalidArgumentsException iae)
    	{
    		throw new InvalidArgumentsException("Error: occured in building childrens expression: ",iae);
    	}
    	if( i < 1)
    		throw new InvalidArgumentsException("Error: usage violation, And must be called with at least 1 argument, aborting");
	}
	
	public ItemVariable eval(HashMap<String,ItemVariable> vars) throws InvalidArgumentsException
	{
		Vector<ItemVariable> ans = new Vector<ItemVariable>();
		//evaluate all children
		try{
			for(int i=0; i< subparts.size(); i++)
			{
				ans.add(subparts.elementAt(i).eval(vars));
			}
		}catch(InvalidArgumentsException iae)
		{
			iae.printStackTrace();
			throw new InvalidArgumentsException("Error: occured in sub Node evaluation within 'and'",iae);
		}
		
		//enforce all child responses are of type Boolean
		for(int i=0; i<ans.size(); i++)
		{
			if(ans.elementAt(i) != null)
			{
				if((ans.elementAt(i).varType != BaseType.bool) || (ans.elementAt(i).card != Cardinality.single))
					throw new InvalidArgumentsException("Error: 'and' operation can only be performed on all 'boolean' type sub children with single cardinality, aborting");
			}
		}
		
		//evaluate children and return a new Boolean
		Boolean retVal = new Boolean(true);
		int nCount= 0;
		for(int i=0; i<ans.size(); i++)
		{
			if(ans.elementAt(i) == null)
				nCount++;
			if(!((Boolean)ans.elementAt(i).value).booleanValue())
			{
				retVal = new Boolean(false);
			}
		}
		if(nCount > 0 && retVal.booleanValue())
			return null;
		return new TransitoryVariable(BaseType.bool,Cardinality.single,retVal);
	}

}
