/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression.operators;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.jdom.Element;

import r2q2.processing.expression.Expression;
import r2q2.processing.expression.ExpressionFactory;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.processing.util.OrderableVector;
import r2q2.variable.ItemVariable;
import r2q2.variable.ItemVariable.Cardinality;
import r2q2.variable.ItemVariable.BaseType;
import r2q2.variable.TransitoryVariable;

/**
 * 
 * Delete operator expression
 * 
 * constraints: 
 *   takes a two sub expressions of the same base type
 *   first sub expression must have single cardinality
 *   second sub expression must be a container (multiple or ordered)
 *   result is a new container derived by removing all instances of the first
 *   expression from the second
 */
public class Delete implements Expression{
	
	private Expression subpart[];
	
	public Delete(Element _e) throws InvalidArgumentsException
	{
		subpart = new Expression[2];
		//get from the factory an expression for each child element (Node)
		ExpressionFactory of = ExpressionFactory.getInstance();
		Iterator children = _e.getChildren().iterator();
    	int i=0;
		try{
			while(children.hasNext()){
	    		if(i < 2)
	    		{
					subpart[i] = of.makeExpression((Element)children.next());
	    		}
		    	i++;
	    	}
    	}catch(InvalidArgumentsException iae)
    	{
    		throw new InvalidArgumentsException("Error occured in building childrens expression: ",iae);
    	}
    	if( i < 2 || i > 2)
    		throw new InvalidArgumentsException("Error: usage violation, Member must be called with exactly 2 argument rather than the "+i+" arguments specified");
	}
	
	public ItemVariable eval(HashMap<String,ItemVariable> vars) throws InvalidArgumentsException
	{
		try{
			ItemVariable test1 = subpart[0].eval(vars);
			ItemVariable test2 = subpart[1].eval(vars);
			
			if (test1.card == Cardinality.single && (test2.card == Cardinality.multiple || test2.card == Cardinality.ordered))
			{
				
				Vector checkVector = (Vector)test2.value;
				Vector retVec = new Vector();
				//check to make sure that types are compatible
				if(test1.varType == test2.varType)
				{
					for(int i=0; i<checkVector.size(); i++)
					{
						if(!checkVector.elementAt(i).equals(test1.value))
							retVec.add(checkVector.elementAt(i));
					}
					return new TransitoryVariable(test1.varType,test2.card,retVec);
				}else
					throw new InvalidArgumentsException("Error: arguements of differing or constrain violating type, aborting");
			}else
				throw new InvalidArgumentsException("Error: cannot perform a random selection from a non-container target");
		}
		catch(InvalidArgumentsException iae)
		{
			throw new InvalidArgumentsException("Error: evaluation of container yielded unexpected exception",iae);
		}
	}
}