/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression.operators;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.jdom.Element;

import r2q2.processing.expression.Expression;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.processing.expression.ExpressionFactory;
import r2q2.processing.util.OrderableVector;
import r2q2.variable.ItemVariable;
import r2q2.variable.TransitoryVariable;
import r2q2.variable.ItemVariable.Cardinality;

/**
 * 
 * Index operator expression
 * 
 * constraints: 
 *   1 child expression only
 *   returns the 'nth' index from a container
 *   indexing starts from 1, NOT 0
 *   must be on a container
 *   returns null if index exceeds container length
 */
public class Index implements Expression {

	private Expression subpart;
	private int n;
	
	public Index(Element _e) throws InvalidArgumentsException
	{
		//get from the factory an expression for each child element (Node)
		ExpressionFactory of = ExpressionFactory.getInstance();
		Iterator children = _e.getChildren().iterator();
		
		n = Integer.parseInt(_e.getAttributeValue("n"));
    	
		int i = 0;
		try{
			while(children.hasNext()){
	    		subpart = of.makeExpression((Element)children.next());
	    		i++;
	    	}
    	}catch(InvalidArgumentsException iae)
    	{
    		throw new InvalidArgumentsException("Error: occured in building childrens expression: ",iae);
    	}
    	if( i < 1 || i > 1)
    		throw new InvalidArgumentsException("Error: usage violation, IsNull must be called with exactly 1 argument rather than the "+i+" arguments specified");
	}
	
	public ItemVariable eval(HashMap<String,ItemVariable> vars) throws InvalidArgumentsException
	{
		ItemVariable ans = null;
		try{
			ans = subpart.eval(vars);
		}catch(InvalidArgumentsException iae)
		{
			throw new InvalidArgumentsException("Error: occured in sub Node evaluation within IsNull",iae);
		}
		if(ans.card == Cardinality.ordered)
		{
			if (n > ((Vector)ans.value).size())
				return null;
			return new TransitoryVariable(ans.varType,Cardinality.single,((Vector)ans.value).elementAt(n));
		}else
			throw new InvalidArgumentsException("Error: can only perform index on an 'ordered' container type instance");
	}

}
