/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression.operators;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.jdom.Element;

import r2q2.processing.expression.Expression;
import r2q2.processing.expression.ExpressionFactory;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.processing.util.OrderableVector;
import r2q2.variable.ItemVariable;
import r2q2.variable.TransitoryVariable;
import r2q2.variable.ItemVariable.BaseType;
import r2q2.variable.ItemVariable.Cardinality;

/**
 * 
 * integer modulus operator expression
 * 
 * constraints: 
 *   takes two sub expressions of Integer base type
 *   sub expressions must have single cardinality
 *   returns a single integer: the remainder of the first expression divided by the second
 *   if either sub expression is NULL, NULL is returned
 *   if the second sub expression is 0 NULL is returned
 */
public class IntegerModulus implements Expression{
	
	private Expression subpart[];
	
	public IntegerModulus(Element _e) throws InvalidArgumentsException
	{
		subpart = new Expression[2];
		//get from the factory an expression for each child element (Node)
		ExpressionFactory of = ExpressionFactory.getInstance();
		Iterator children = _e.getChildren().iterator();
    	int i=0;
		try{
			while(children.hasNext()){
	    		if(i < 2)
	    		{
					subpart[i] = of.makeExpression((Element)children.next());
	    		}
		    	i++;
	    	}
    	}catch(InvalidArgumentsException iae)
    	{
    		throw new InvalidArgumentsException("Error occured in building childrens expression: ",iae);
    	}
    	if( i < 2 || i > 2)
    		throw new InvalidArgumentsException("Error: usage violation, 'modulus' must be called with exactly 2 argument rather than the "+i+" arguments specified");
	}
	
	public ItemVariable eval(HashMap<String,ItemVariable> vars) throws InvalidArgumentsException
	{
		try{
			ItemVariable test1 = subpart[0].eval(vars);
			ItemVariable test2 = subpart[1].eval(vars);
			
			if(test1 == null || test2 == null)
				return null;
			if(test1.card != Cardinality.single || test2.card != Cardinality.single)
				throw new InvalidArgumentsException("Error: can only perform 'integer divide' on single cardinality arguements, aborting");
			
			if(test1.varType == BaseType.integer && test2.varType == BaseType.integer)
			{
				int i1;
				int i2;
				//make all into doubles for comparison
				
				i1 = ((Integer)test1.value).intValue();
				i2 = ((Integer)test2.value).intValue();
				
				if(i2 == 0)
					return null;
				return new TransitoryVariable(BaseType.integer,Cardinality.single,new Integer(i1%i2));
			}else
				throw new InvalidArgumentsException("Error: non-numeric, or unsupported numeric types being used, aborting");
		}
		catch(InvalidArgumentsException iae)
		{
			throw new InvalidArgumentsException("Error: evaluation of container yielded unexpected exception",iae);
		}
	}
}
