/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression.operators;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.jdom.Element;

import r2q2.processing.expression.Expression;
import r2q2.processing.expression.ExpressionFactory;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.processing.util.OrderableVector;
import r2q2.variable.ItemVariable;
import r2q2.variable.TransitoryVariable;
import r2q2.variable.ItemVariable.BaseType;
import r2q2.variable.ItemVariable.Cardinality;

/**
 * 
 * Match operator expression
 * 
 * constraints: 
 *   takes a two sub expressions of the same base type, single cardinality
 *   result is true if the two objects represent the same value
 *   null is returned if either of the expressions is null
 *   should not be used with float types
 *   *must* not be used with duration types
 */
public class Match implements Expression{
	
	private Expression subpart[];
	
	public Match(Element _e) throws InvalidArgumentsException
	{
		subpart = new Expression[2];
		//get from the factory an expression for each child element (Node)
		ExpressionFactory of = ExpressionFactory.getInstance();
		Iterator children = _e.getChildren().iterator();
    	int i=0;
		try{
			while(children.hasNext()){
	    		if(i < 2)
	    		{
					subpart[i] = of.makeExpression((Element)children.next());
					System.out.println("finished making expression ["+i+"]");
	    		}
		    	i++;
	    	}
			System.out.println("[MATCH] finished evaluating child nodes");
    	}catch(InvalidArgumentsException iae)
    	{
    		throw new InvalidArgumentsException("Error occured in building childrens expression: ",iae);
    	}
    	if( i < 2 || i > 2)
    		throw new InvalidArgumentsException("Error: usage violation, 'match' must be called with exactly 2 argument rather than the "+i+" arguments specified");
	}
	
	public ItemVariable eval(HashMap<String,ItemVariable> vars) throws InvalidArgumentsException
	{
		ItemVariable test1 = null;
		ItemVariable test2 = null;
		
		try{
			test1 = subpart[0].eval(vars);
			test2 = subpart[1].eval(vars);
		}
		catch(InvalidArgumentsException iae)
		{
			throw new InvalidArgumentsException("Error: evaluation of children yielded unexpected exception",iae);
		}
		
		if(test1 == null || test2 == null)
		{
			System.out.println("test1 = "+test1+" test2 = "+test2);
			return null;
		}
		if(test1.value == null || test2.value == null)
			return null;
		System.out.println("test2.card " + test2.card + " || test1.card " + test1.card);
		if(test1.card != test2.card)
			throw new InvalidArgumentsException("Error: both arguements must be of the same cardinality, aborting");
		
		if(test1.varType == BaseType.afloat || test1.varType == BaseType.duration)
			throw new InvalidArgumentsException("Error: cannot perform 'match' on float (implementation specific, use equal) or duration types, aborting");
		
		System.out.println("[MATCH] performing a match between arg1 : "+test1.value+" and arg 2 : "+test2.value);
		
		if(test1.value.equals(test2.value))
			return new TransitoryVariable(BaseType.bool,Cardinality.single,new Boolean(true));
		else
			return new TransitoryVariable(BaseType.bool,Cardinality.single,new Boolean(false));
	}
}