/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression.operators;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.jdom.Element;

import r2q2.processing.expression.Expression;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.processing.expression.ExpressionFactory;
import r2q2.processing.util.GenericObjectComparator;
import r2q2.processing.util.OrderableVector;
import r2q2.variable.ItemVariable;
import r2q2.variable.TransitoryVariable;
import r2q2.variable.ItemVariable.BaseType;
import r2q2.variable.ItemVariable.Cardinality;

/**
 * 
 * Ordered operator expression
 * 
 * constraints: 
 *   0 or more sub-expressions, single or multiple cardinality
 *   all sub expressions of the same base type
 *   creates a list {a,b,c,d} of elements
 *   lists cannot be nested, ie no {{a,b},c,d}, etc
 *   orders the container into ascending order, or best approximation
 *   by bytes for all other types
 */
public class Ordered implements Expression {

	private Vector<Expression> subparts = new Vector<Expression>();
	
	public Ordered(Element _e) throws InvalidArgumentsException
	{
		//get from the factory an expression for each child element (Node)
		ExpressionFactory of = ExpressionFactory.getInstance();
		Iterator children = _e.getChildren().iterator();
    	try{
			while(children.hasNext()){
	    		subparts.add(of.makeExpression((Element)children.next()));
	    	}
    	}catch(InvalidArgumentsException iae)
    	{
    		throw new InvalidArgumentsException("Error occured in building childrens expression: ",iae);
    	}
	}
	
	public ItemVariable eval(HashMap<String,ItemVariable> vars) throws InvalidArgumentsException
	{
		Vector<Object> container = new Vector<Object>();
		BaseType typeCheck = null;
		try{
			for(int i=0; i<subparts.size(); i++)
			{
				ItemVariable test = subparts.elementAt(i).eval(vars);
				if(test != null)
				{
					if(test.card != Cardinality.single && test.card != Cardinality.ordered)
						throw new InvalidArgumentsException("Error: ordered can only be conducted on expressions of single or ordered cardinality, aborting");
					if(typeCheck == null)
						typeCheck = test.varType;
					else if(test.varType != typeCheck)
						throw new InvalidArgumentsException("Error: ordered can only be conducted on elements of the same base type, aborting");
					if(test != null)
					{
						if (test.card == Cardinality.ordered  && test.value instanceof Vector)
						{
							Vector innerTest = (Vector)test.value;
							for(int j=0; j< innerTest.size(); j++)
							{
								container.add(innerTest.elementAt(j));
							}
						}else
							container.add(test.value);
					}
				}
			}
		}
		catch(InvalidArgumentsException iae)
		{
			throw new InvalidArgumentsException("Error: evaluation of container yielded unexpected exception",iae);
		}
		//check that all elements are of the same class type
		if(container.size() == 0)
			return null;
		System.out.println("[ordered] start of container blurb--");
		for(int i=0; i < container.size(); i++)
			System.out.println("["+i+"] is : "+container.elementAt(i));
		System.out.println("[ordered] end of container blurb--");
		//Collections.sort(container,new GenericObjectComparator());
		return new TransitoryVariable(typeCheck,Cardinality.ordered,container);
	}

}
