/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression.operators;

import java.util.HashMap;
import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jdom.Element;

import r2q2.processing.expression.Expression;
import r2q2.processing.expression.ExpressionFactory;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.variable.ItemVariable;
import r2q2.variable.TransitoryVariable;
import r2q2.variable.ItemVariable.BaseType;
import r2q2.variable.ItemVariable.Cardinality;

/**
 * 
 * 'patternMatch' operator expression
 * 
 * constraints: 
 *   takes a single expression of the base type 'string' with single cardinality
 *   returns Boolean 'true' if the expression matches the regular expression passed
 *   in as an attribute
 *   if the sub expression is NULL, NULL is returned
 *   
 *   NOTE this is only standard regEx guaranteed compatibility, the specification has
 *   not been fully compared to that in the w3 XML 2 specification
 */

public class PatternMatch implements Expression{
	
	private Expression subpart;
	private String regExp = "";
	
	public PatternMatch(Element _e) throws InvalidArgumentsException
	{
		//get from the factory an expression for each child element (Node)
		ExpressionFactory of = ExpressionFactory.getInstance();
		Iterator children = _e.getChildren().iterator();
		regExp = _e.getAttributeValue("pattern");
    	int i=0;
		try{
			while(children.hasNext()){
	    		subpart = of.makeExpression((Element)children.next());
		    	i++;
	    	}
    	}catch(InvalidArgumentsException iae)
    	{
    		throw new InvalidArgumentsException("Error occured in building childrens expression: ",iae);
    	}
    	if( i > 1 )
    		throw new InvalidArgumentsException("Error: usage violation, 'patternMatch' can only be called with 1 argument, aborting");
	}
	
	public ItemVariable eval(HashMap<String,ItemVariable> vars) throws InvalidArgumentsException
	{
		try{
			ItemVariable test = subpart.eval(vars);
			if(test == null)
				return null;
			if (test.varType == BaseType.string)
			{
				String str = (String)test.value;
				
				Pattern pattern = Pattern.compile(regExp);
				Matcher matcher = pattern.matcher(str);
				
				if(matcher.find())
					return new TransitoryVariable(BaseType.bool,Cardinality.single, new Boolean(true));
				else
					return new TransitoryVariable(BaseType.bool,Cardinality.single, new Boolean(false));
			}else 
				throw new InvalidArgumentsException("Error: patternMatch must be provided a single cardinality string in order to function, aborting");
		}
		catch(InvalidArgumentsException iae)
		{
			throw new InvalidArgumentsException("Error: evaluation of children yielded unexpected exception",iae);
		}
	}
}