/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.processing.expression.operators;

import java.util.HashMap;
import java.util.Iterator;

import org.jdom.Element;

import r2q2.processing.expression.Expression;
import r2q2.processing.expression.ExpressionFactory;
import r2q2.processing.expression.InvalidArgumentsException;
import r2q2.variable.ItemVariable;
import r2q2.variable.TransitoryVariable;
import r2q2.variable.ItemVariable.BaseType;
import r2q2.variable.ItemVariable.Cardinality;

/**
 * 
 * 'stringMatch' operator expression
 * 
 * constraints: 
 *   takes a two sub expressions of the base type 'string'
 *   expressions must have single cardinality
 *   returns Boolean 'true' if the first expression is the same as the second
 *   can also be called as a substring method
 *   if sub expression is NULL, NULL is returned
 */

public class StringMatch implements Expression{
	
	private Expression subpart[];
	private boolean caseSensitive;
	private boolean substring;
	
	public StringMatch(Element _e) throws InvalidArgumentsException
	{
		subpart = new Expression[2];
		//get from the factory an expression for each child element (Node)
		ExpressionFactory of = ExpressionFactory.getInstance();
		Iterator children = _e.getChildren().iterator();
		if(_e.getAttributeValue("caseSensitive") != null)
			caseSensitive = Boolean.parseBoolean(_e.getAttributeValue("caseSensitive"));
		else
			caseSensitive = new Boolean(true);
		if(_e.getAttributeValue("substring") != null)
			substring = Boolean.parseBoolean(_e.getAttributeValue("substring"));
		else
			substring = new Boolean(false);
    	int i=0;
		try{
			while(children.hasNext()){
	    		if(i < 2)
	    		{
					subpart[i] = of.makeExpression((Element)children.next());
	    		}
		    	i++;
	    	}
    	}catch(InvalidArgumentsException iae)
    	{
    		throw new InvalidArgumentsException("Error occured in building childrens expression: ",iae);
    	}
    	if( i < 2 || i > 2)
    		throw new InvalidArgumentsException("Error: usage violation, 'stringMatch' must be called with exactly 2 argument rather than the "+i+" arguments specified");
	}
	
	public ItemVariable eval(HashMap<String,ItemVariable> vars) throws InvalidArgumentsException
	{
		try{
			ItemVariable test1 = subpart[0].eval(vars);
			ItemVariable test2 = subpart[1].eval(vars);
			
			if(test1 == null || test2 == null)
			{
				return null;
			}
			
			//cardinality check
			if(test1.card != Cardinality.single || test2.card != Cardinality.single)
				throw new InvalidArgumentsException("Error: can only take two, single cardinality strings as arguements, aborting");
			
			if (test1.varType == BaseType.string && test2.varType == BaseType.string)
			{
				String s1 = (String)test1.value;
				String s2 = (String)test2.value;
				
				if(!caseSensitive)
				{
					s1.toUpperCase();
					s2.toUpperCase();
				}
				
				if(substring)
				{
					if(s2.indexOf(s1) > -1)
						return new TransitoryVariable(BaseType.bool,Cardinality.single,new Boolean(true));
					return new TransitoryVariable(BaseType.bool,Cardinality.single,new Boolean(false));
				}else
				{
					if(s2.equals(s1))
						return new TransitoryVariable(BaseType.bool,Cardinality.single,new Boolean(true));
					return new TransitoryVariable(BaseType.bool,Cardinality.single,new Boolean(false));
				}
			}else
				throw new InvalidArgumentsException("Error: stringMatch must be provided two single cardinality string types in order to function, aborting");
		}
		catch(InvalidArgumentsException iae)
		{
			throw new InvalidArgumentsException("Error: evaluation of children yielded unexpected exception",iae);
		}
	}
}