/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.rendering;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.activation.DataHandler;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.SOAPException;

import org.apache.axis.Message;
import org.apache.axis.MessageContext;

import r2q2.NoAttachmentException;
import r2q2.SpecificationViolationException;
import r2q2.util.streams.ByteArrayDataSource;
import r2q2.util.streams.StreamCopier;

/**
 * TODO javadoc
 * @author Gavin Willingham
 *
 */
public class RenderingEngine {
	
	public static final String XHTML = "r2q2.rendering.xhtml.XHTMLRenderer";
	
	public RenderingEngineResponse render(HashMap[] _variables, 
									HashMap _options,
									List<String> _variableNames,
									boolean _firstStage,
									String _format)
									throws RenderingException, SpecificationViolationException{
		try{
		MessageContext msgContext = MessageContext.getCurrentContext();
        Message request = msgContext.getRequestMessage();
        Message response = msgContext.getResponseMessage();
        Iterator attachmentIter = request.getAttachments();
        
        //Convert the attachment to a byte array
        if(!attachmentIter.hasNext()) throw new RenderingException(
        										new NoAttachmentException("XML not supplied to Rendering Engine"));
		AttachmentPart ap = (AttachmentPart) attachmentIter.next();
        byte[] xml = null;
        try {
			xml = StreamCopier.copyToByteArray(ap.getDataHandler().getInputStream());
		} catch (IOException e) {
			throw new RenderingException("IOException when converting attachment to byte array", e);
		} catch (SOAPException e) {
			throw new RenderingException("SOAPException when converting attachment to byte array", e);
		}
        
		//build the renderer object
		Renderer rend = null;
		try {
			rend = RendererFactory.getRenderer(_format);
		} catch (UnsupportedFormatException e) {
			throw new RenderingException(
					new UnsupportedFormatException("Format " + _format + " is not valid!"));
		}
		
		
		//Convert arrays to lists
		HashMap <String, List<List<String>>> opts = null;
		if(_options != null){
			opts = new HashMap<String, List<List<String>>>();
			Iterator<String> iter = _options.keySet().iterator();
			while(iter.hasNext()){
				String key = iter.next();
				Object nextList =  _options.get(key);
				if(nextList instanceof List){
					opts.put(key, (List)nextList);
				}
				else{
					throw new RenderingException("Error converting web-service sent opts list: type is " + nextList.getClass().getName());
				}
			}
		}
		
		RendererResponse out_ = rend.render(xml, _variables, opts, _variableNames, _firstStage);
		
		//build the attachment
		byte[] attachmentBytes = out_.getContent();
		AttachmentPart ap2 = response.createAttachmentPart();
		ap2.setDataHandler(
				new DataHandler(
						new ByteArrayDataSource("output", attachmentBytes, rend.mimeType)));
		response.addAttachmentPart(ap2);
		
		return out_.getResp();
	}
	catch(Throwable t){
		t.printStackTrace();
		throw new RenderingException("JJKJKL");
	}
	}
}
