/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.rendering.ws;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.activation.DataHandler;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.SOAPException;

import org.apache.axis.AxisFault;

import com.gavinwillingham.util.properties.NoSuchPropertyException;
import com.gavinwillingham.util.properties.PropertiesLoader;

import r2q2.NoAttachmentException;
import r2q2.SpecificationViolationException;
import r2q2.rendering.RenderingEngineResponse;
import r2q2.rendering.RenderingException;
import r2q2.router.RenderingResponse;
import r2q2.util.streams.ByteArrayDataSource;
import r2q2.util.streams.StreamCopier;

public class RenderingEngineClient {

	URL location;
	
	public RenderingEngineClient (String _location) throws RenderingException{
		try{
			location = new URL(_location);
		}
		catch(MalformedURLException mfue){
			throw new RenderingException("Invalid URL for Rendering web service (" + _location + ")");
		}
	}
	
	public RenderingEngineResponse render(byte[] _xml, HashMap[] _vars, HashMap _opts, List _varNames, boolean _firstStage, String _format, RenderingResponse _reResp) throws RenderingException, SpecificationViolationException{
		//Call the rendering service
    	RenderingEngineServiceLocator loc = new RenderingEngineServiceLocator();
    	RenderingEngineSoapBindingStub stub = null;
		try {
			stub = new RenderingEngineSoapBindingStub(location, loc);
		} catch (AxisFault e) {
			throw new RenderingException("Error creating rendering engine stub", e);
		}
    	stub.addAttachment(
    			new DataHandler(
    				new ByteArrayDataSource("question.xml", _xml, "text/xml")));
    	
    	r2q2.rendering.ws.RenderingEngineResponse resp = null;
    	
    	Object[] reqVarNames = null;
    	if(_varNames != null) reqVarNames = _varNames.toArray();
    	
    	try {
			resp = stub.render(_vars, _opts, reqVarNames, _firstStage, _format);
		} catch (r2q2.rendering.ws.RenderingException e) {
			throw new RenderingException(e);
		} catch (r2q2.rendering.ws.SpecificationViolationException e) {
			throw new SpecificationViolationException(e);
		} catch (RemoteException e) {
			throw new RenderingException("RemoteException connecting to rendering web service");
		}
    	   	
		List<String> listVarNames = new ArrayList<String>();
		Object[] respVarNames = resp.getVarNames();
		if(respVarNames != null){
			try{
				for(int i=0; i<respVarNames.length; i++) listVarNames.add((String)respVarNames[i]);
			}
			catch(ClassCastException e){
				throw new RenderingException("Couldn't cast a variable name to a string");
			}
		}
		
		//extract the content and return
    	Object[] attachments = stub.getAttachments();
    	if(attachments.length == 0)
			throw new RenderingException("Renderer did not return any output", new NoAttachmentException("No content returned from renderer"));

    	AttachmentPart ap = (AttachmentPart) attachments[0];
    	byte[] file;
		try {
			file = StreamCopier.copyToByteArray(ap.getDataHandler().getInputStream());
		} catch (Exception e) {
			throw new RenderingException("Error copying rendering output to byte array", e);
		}
    	_reResp.file = file;
    	
    	RenderingEngineResponse out_ = new RenderingEngineResponse(listVarNames, resp.getOrder());
    	return out_;
	}
}
