/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.rendering.xhtml.processors;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.jdom.Attribute;
import org.jdom.Element;

import r2q2.SpecificationViolationException;
import r2q2.rendering.RenderingException;
import r2q2.rendering.xhtml.UnsupportedTagException;
import r2q2.rendering.xhtml.XHTMLRenderer;
import r2q2.rendering.xhtml.structure.xhtml.XHTMLObject;

public class GraphicGapMatchInteractionProcessor extends GraphicInteractionProcessor {

	//counter for current hotspot
	private int hotspotNum = 0;
	//number of hotspots
	private int hotspotSize = 0;
	
	//counter for current gapImg
	private int gapImgNum = 0;
	//number of gapImgs
	private int gapImgSize = 0;
	
	public GraphicGapMatchInteractionProcessor(XHTMLRenderer re) throws RenderingException {
		super(re);
		supportedTags.add("graphicGapMatchInteraction");
		supportedTags.add("gapImg");
		supportedTags.add("associableHotspot");
		supportedTags.add("object");
	}

	@Override
	public void process(Element _e) throws UnsupportedTagException, RenderingException, SpecificationViolationException {
		String elementName = _e.getName();
		
		if(!supportedTags.contains(elementName))
			throw new UnsupportedTagException(elementName + " in graphicGapMatchinteraction");
		
		if(elementName.equals("graphicGapMatchInteraction"))
			processgraphicOrderInteraction(_e);
		else if(elementName.equals("associableHotspot"))
			processAssociableHotspot(_e);
		else if(elementName.equals("gapImg"))
			processGapImg(_e);
	}
	
	private void processgraphicOrderInteraction(Element _e) throws UnsupportedTagException, RenderingException, SpecificationViolationException {
		
    	String attname = "";
		try{
			//Now process all of the attributes in the array 
	        Iterator<Attribute> attrs = _e.getAttributes().iterator();
	        while (attrs.hasNext()){          
	            Attribute attr = attrs.next();                  
	            attname = attr.getName();                  
	            String value = attr.getValue();  
	            if(attname.equals("responseIdentifier"))
	                responseIdentifier = value;
	        }
		}
	    catch(NumberFormatException e){
	    	throw new SpecificationViolationException(attname + " could not be converted to the appropriate type", e);
	    }
		
        //test that response variable is identifier
        HashMap<String, HashMap> responseVars = engine.vars[0];
    	HashMap interactionResponse = responseVars.get(responseIdentifier);
    	if(interactionResponse == null)
    		throw new SpecificationViolationException("ResponseVar " + responseIdentifier + " not present");
    	String baseType = (String) interactionResponse.get("baseType");
    	if(!baseType.equals("directedPair"))
    		throw new SpecificationViolationException("Cannot assign hotspots to a variable (" + responseIdentifier + ") that is not of base type directedPair in graphicGapMatchInteraction");
    	String cardinality = (String) interactionResponse.get("cardinality");
    	if(!cardinality.equals("multiple"))
    		throw new SpecificationViolationException("Cannot assign hotspots in graphicOrderInteraction to a variable (" + responseIdentifier + ") that is not multiple cardinality in graphicGapMatchInteraction");
    	
    	//get any previous values, i.e. user response
    	response = (Vector) interactionResponse.get("candidateValues");
		
		//output the prompt
    	Element prompt = _e.getChild("prompt", XHTMLRenderer.ns);
    	if(prompt != null) engine.processElement(prompt);
    	
    	//find the object node
    	Element object = _e.getChild("object", XHTMLRenderer.ns);
    	if(object == null)
    		throw new SpecificationViolationException("No object found under graphicGapMatchInteraction");
    	XHTMLObject obj = processObject(object);
	    
    	//find the hotspot nodes
    	List<Element> hotspots = _e.getChildren("associableHotspot", engine.ns);
    	if(hotspots == null || hotspots.size() < 1)
    		throw new SpecificationViolationException("No hotspots beneath graphicGapMatchInteraction");
    	hotspotSize = hotspots.size();
    	
    	//find the gapImg nodes
    	List<Element> gapImgs = _e.getChildren("gapImg", engine.ns);
    	if(gapImgs == null || gapImgs.size() < 1)
    		throw new SpecificationViolationException("No gapImgs beneath graphicGapMatchInteraction");
    	gapImgSize = gapImgs.size();
    	
	    //start the applet
	    engine.addToOutput("<applet"); 
        engine.addToOutput("codebase='" + basePath + "'", false);
    	engine.addToOutput("code = 'BoundedGraphicalApplet.class'", false);
    	engine.addToOutput("width = '" + obj.getWidth() + "'", false);
    	engine.addToOutput("height = '" + (obj.getHeight() + 40) + "'", false);
    	engine.addToOutput("hspace = '0'", false);
    	engine.addToOutput("vspace = '0'", false);
    	engine.addToOutput("align = 'middle'", false);
    	engine.addToOutput(">", false);
    	engine.addToOutput("<param name ='identifier' value = '" + responseIdentifier + "'>", false);
    	engine.addToOutput("<param name ='operation_mode' value = 'gap_match_interaction'>", false);
    	engine.addToOutput("<param name ='number_of_responses' value ='" + hotspotSize + "'>", false);
    	engine.addToOutput("<param name ='baseType' value ='directedPair'>", false);
    	
    	String imagePath = obj.getData();
    	if(imagePath.startsWith(basePath))
    		imagePath = imagePath.substring(basePath.length());
    	engine.addToOutput("<param name ='background_image' value ='" + imagePath + "'>", false);
    	
    	//process the hotspots
    	engine.addToOutput("<param name ='hotspot_count' value ='" + hotspotSize + "'>", false);
    	process(hotspots);
    	engine.addToOutput("<param name ='movable_element_count' value ='" + gapImgSize + "'>", false);
    	process(gapImgs);
    	
    	//add any previous selections
    	if(response != null && response.size() > 0){
    		String feedback = "";
    		int i=0;
    		for(; i<response.size() - 1; i++){
    			feedback += response.get(i) + ",";
    		}
    		feedback += response.get(i);
    		engine.addToOutput("<param name='feedback' value='" + feedback + "' />", false);
    	}
    	
    	//close the applet
    	engine.decreaseDepth();
    	engine.addToOutput("</applet>", false);
	}
	
	private void processAssociableHotspot(Element _e) throws SpecificationViolationException {
		//extract the attributes
		String id = _e.getAttributeValue("identifier");
		if(id == null)
			throw new SpecificationViolationException("No identifier attribute for hotspot choice");
		String shape = _e.getAttributeValue("shape");
		if(shape == null)
			throw new SpecificationViolationException("No shape attribute for hotspot choice");
		String coords = _e.getAttributeValue("coords");
		if(coords == null)
			throw new SpecificationViolationException("No coords attribute for hotspot choice");
		String label = _e.getAttributeValue("hotspotLabel");
		String matchGroup = _e.getAttributeValue("matchGroup");
		String matchMax = _e.getAttributeValue("matchMax");
		
		//build the output
		String temp = "<param name='hotspot" + hotspotNum++ + "' " +
				"value='" + id + "" +
				"::" +
				"::" + shape +
				"::" + coords; 
		
		if(label != null)
			temp += "::hotSpotLabel:" + label.trim();
		if(matchGroup != null){
			String[] matches = matchGroup.split(",");
			for(int i=0; i<matches.length; i++)
				temp += "::" + matches[i];
		}
		
		if(matchMax != null)
			temp +="::maxAssociations:"+Integer.parseInt(matchMax);
		
		temp += "' />";
				
		engine.addToOutput(temp, false);
	}
	
	private void processGapImg(Element _e) throws SpecificationViolationException, RenderingException{
		//extract the attributes
		String id = _e.getAttributeValue("identifier");
		if(id == null)
			throw new SpecificationViolationException("No identifier attribute for gapImg");
		String label = _e.getAttributeValue("objectLabel");
		String matchGroup = _e.getAttributeValue("matchGroup");
		String matchMax = _e.getAttributeValue("matchMax");
		
		//build the output
		String temp = "<param name='movable_object" + gapImgNum++ + "' " +
				"value='" + id + "" +
				"::";
		
		//get the image name
    	Element object = _e.getChild("object", XHTMLRenderer.ns);
    	if(object == null)
    		throw new SpecificationViolationException("No object found under graphicGapMatchInteraction");
    	XHTMLObject obj = processObject(object);
    	String imageName = obj.getData();
    	if(imageName.startsWith(basePath))
    		imageName = imageName.substring(basePath.length());
    	temp += imageName;
		
		if(label != null)
			temp += "::hotSpotLabel:" + label.trim();
		if(matchGroup != null){
			String[] matches = matchGroup.split(",");
			for(int i=0; i<matches.length; i++)
				temp += "::" + matches[i];
		}
		
		if(matchMax != null)
			temp +="::maxAssociations:"+Integer.parseInt(matchMax);
		
		temp += "' />";
				
		engine.addToOutput(temp, false);
	}

}
