/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.rendering.xhtml.processors;

import java.io.IOException;
import java.util.HashSet;
import java.util.Vector;

import org.jdom.Element;

import com.gavinwillingham.util.properties.NoSuchPropertyException;
import com.gavinwillingham.util.properties.PropertiesLoader;

import r2q2.SpecificationViolationException;
import r2q2.rendering.RenderingException;
import r2q2.rendering.xhtml.UnsupportedTagException;
import r2q2.rendering.xhtml.XHTMLRenderer;
import r2q2.rendering.xhtml.processors.util.ObjectProcessor;
import r2q2.rendering.xhtml.structure.xhtml.XHTMLObject;
import r2q2.util.strings.TokenUtils;

public abstract class GraphicInteractionProcessor extends InteractionProcessor {
	protected HashSet<String> supportedObjectTypes = null;
	protected String basePath = "";

	protected GraphicInteractionProcessor(XHTMLRenderer re) throws RenderingException {
		super(re);
		
    	try {
			basePath = PropertiesLoader.getProperty("r2q2.router.content.httpBasePath");
		} catch (Exception e) {
			throw new RenderingException("Error loading htmlBasePath property", e);
		}
		
		supportedTags.add("object");
		loadSupportedObjectTypes();
	}
	
	private void loadSupportedObjectTypes() throws RenderingException {
		if(supportedObjectTypes == null){
			//load the types from properties
			try {
				String props = PropertiesLoader.getProperty("r2q2.rendering.xhtml.graphicInteraction.supportedObjectTypes");
				supportedObjectTypes = TokenUtils.tokensToHashSet(props, " ");
			} catch (IOException e) {
				throw new RenderingException("Couldn't load properties file in graphicInteraction!");
			} catch (NoSuchPropertyException e) {
				throw new RenderingException("Couldn't find property in graphicInteraction!");
			}
		}
	}

	protected XHTMLObject processObject(Element _e) throws RenderingException, SpecificationViolationException{
		//extract the attributes
		String data = _e.getAttributeValue("data");
		if(data == null) 
			throw new SpecificationViolationException("'data' attribute not set in object");
		String type = _e.getAttributeValue("type");
		if(type == null) 
			throw new SpecificationViolationException("'type' attribute not set in object");
		
		ObjectProcessor op = new ObjectProcessor(supportedObjectTypes);
		
		return op.processObject(_e);
	}

}
