/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.rendering.xhtml.processors;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.jdom.Attribute;
import org.jdom.Content;
import org.jdom.Element;

import r2q2.SpecificationViolationException;
import r2q2.rendering.RenderingException;
import r2q2.rendering.xhtml.UnsupportedTagException;
import r2q2.rendering.xhtml.XHTMLRenderer;

public class HottextInteractionProcessor extends InteractionProcessor {

	private int maxChoices;
	private String cardinality;
	private Vector<String> candidateValues;
	private boolean radio;
	
	public HottextInteractionProcessor(XHTMLRenderer re) throws RenderingException {
		super(re);

		supportedTags.add("hottextInteraction");
		supportedTags.add("hottext");
	}

	@Override
	public void process(Element _e) throws UnsupportedTagException,
			RenderingException, SpecificationViolationException {
		
		String elementName = _e.getName();
		
		if(!supportedTags.contains(elementName)){
			throw new UnsupportedTagException(elementName + " in hotspotInteraction");
		}
		else if(_e.getName().equals("hottextInteraction")){
			processHottextInteraction(_e);
		}
		else if(_e.getName().equals("hottext")){
			processHottext(_e);
		}
	}

	private void processHottext(Element _e) throws SpecificationViolationException, UnsupportedTagException, RenderingException{
		// TODO Auto-generated method stub
		engine.addToOutput("<span class='hottext'>");
		String id = _e.getAttributeValue("identifier");
		boolean checked = false;
		
		if(radio)
		{
			if(candidateValues != null && candidateValues.elementAt(0).equals(id))
				checked = true;
		}else
		{
			if(candidateValues != null)
			{
				for(int i=0; i < candidateValues.size(); i++)
					if(candidateValues.elementAt(i).equals(id))
						checked = true;
			}
		}
		String temp = "";
		temp = "<input type="+(radio? "'radio'":"'checkbox'")+" name='"+responseIdentifier+"' value='"
					+id+"'"+((checked)?" checked='checked'":"");
		if(maxChoices > 1)
    		temp += "' onclick='limitChecks(" + maxChoices + ", \"" + responseIdentifier + "\", \"" + id + "\")";
		temp += " /> ";
		
		engine.addToOutput(temp,false);
		engine.processElements(_e.getContent());
		engine.decreaseDepth();
		engine.addToOutput("</span>",false);
	}

	private void processHottextInteraction(Element _e) throws SpecificationViolationException, UnsupportedTagException, RenderingException{
		String attname = ""; 
		try{
			//Now process all of the attributes in the array 
	        Iterator<Attribute> attrs = _e.getAttributes().iterator();
	        while (attrs.hasNext()){          
	            Attribute attr = attrs.next();                  
	            attname = attr.getName();                  
	            String value = attr.getValue();  
	            if(attname.equals("responseIdentifier"))
	            	responseIdentifier = value;
	            if(attname.equals("maxChoices"))
	            	maxChoices = Integer.parseInt(value);
	        }
		}
	    catch(NumberFormatException e){
	    	throw new SpecificationViolationException(attname + " could not be converted to the appropriate type", e);
	    }
		
	    HashMap<String, HashMap> responseVars = engine.vars[0];
    	HashMap interactionResponse = responseVars.get(responseIdentifier);
    	if(interactionResponse == null)
    		throw new SpecificationViolationException("ResponseVar " + responseIdentifier + " not present");
    	cardinality = (String) interactionResponse.get("cardinality");
    	
    	candidateValues = (Vector) interactionResponse.get("candidateValues");
	    
    	radio = true;
    	if(maxChoices > 1 || maxChoices == 0)
    	{	
    		radio = false;
    	}
    	
    	if(cardinality.equals("single") && !radio)
    		throw new SpecificationViolationException("cardinality must be multiple in order to have more than one choices");
    	
    	engine.processElements((List<Content>)(List)_e.getChildren());
	}

}
