/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.rendering.xhtml.processors;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.jdom.Attribute;
import org.jdom.Content;
import org.jdom.DataConversionException;
import org.jdom.Element;

import r2q2.SpecificationViolationException;
import r2q2.rendering.RenderingException;
import r2q2.rendering.xhtml.UnsupportedTagException;
import r2q2.rendering.xhtml.XHTMLRenderer;
import r2q2.rendering.xhtml.processors.util.ListElements;

/**
 * TODO javadoc
 * 
 * @author Gavin Willingham
 *
 */
public class InlineChoiceInteractionProcessor extends InteractionProcessor {

	private String responseIdentifier;
	private boolean shuffle = false;
	private String response = null;
	
	public InlineChoiceInteractionProcessor(XHTMLRenderer re) throws RenderingException {
		super(re);
		supportedTags.add("inlineChoiceInteraction");
		supportedTags.add("inlineChoice");
	}

	/* (non-Javadoc)
	 * @see r2q2.rendering.xhtml.InteractionProcessor#process(org.jdom.Element)
	 */
	@Override
	public void process(Element _e) throws UnsupportedTagException, RenderingException, SpecificationViolationException {
		
		String elementName = _e.getName();

		if(!supportedTags.contains(elementName)){
			throw new UnsupportedTagException(elementName + " in inlineChoiceInteraction");
		}
		else if(elementName.equals("inlineChoiceInteraction")){
			processInlineChoiceInteraction(_e);
		}
		else if(elementName.equals("inlineChoice")){
			processInlineChoice(_e);
		}
	}
	
	private void processInlineChoiceInteraction(Element _e) throws SpecificationViolationException, UnsupportedTagException, RenderingException{
		
		String attname = ""; 
		try{
			//Now process all of the attributes in the array 
	        Iterator<Attribute> attrs = _e.getAttributes().iterator();
	        while (attrs.hasNext()){          
	            Attribute attr = attrs.next();                  
	            attname = attr.getName();                  
	            String value = attr.getValue();  
	            if(attname.equals("responseIdentifier"))
	            	responseIdentifier = value;
	            else if(attname.equals("shuffle"))
	                shuffle = Boolean.parseBoolean(value);
	        }
		}
	    catch(NumberFormatException e){
	    	throw new SpecificationViolationException(attname + " could not be converted to the appropriate type", e);
	    }
	    
        //test that response variable is identifier with single cardinality
        HashMap<String, HashMap> responseVars = engine.vars[0];
    	HashMap interactionResponse = responseVars.get(responseIdentifier);
    	if(interactionResponse == null)
    		throw new SpecificationViolationException("ResponseVar " + responseIdentifier + " not present");
    	String baseType = (String) interactionResponse.get("baseType");
    	if(!baseType.equals("identifier"))
    		throw new SpecificationViolationException("Cannot assign choices to a variable (" + responseIdentifier + ") that is not of base type identifier");
    	String cardinality = (String) interactionResponse.get("cardinality");
    	if(!cardinality.equals("single"))
    		throw new SpecificationViolationException("Cannot assign multiple choices to a single cardinality variable");
    	Vector candidateValues = (Vector) interactionResponse.get("candidateValues");
    	if(candidateValues != null) response = (String) candidateValues.get(0);
    	
    	//order the choices
        List<Element> children = _e.getChildren();
        
        if(shuffle){
        	List orderList = engine.opts.get(responseIdentifier);
        	if(engine.opts.get(responseIdentifier) == null)
        	{
        		children = ListElements.randomiseList(children);
        		//set the hashtable
        		List<String> map = ListElements.getOrder(children);
        		List<List<String>> mapList= new Vector<List<String>>();
        		mapList.add(map);
        		engine.opts.put(responseIdentifier, mapList);
        	}else
        	{
        		//need to fetch hashtable
        		children = ListElements.orderList(children,(List<String>)orderList.get(0));
        	}
        }
        
        //start the select box
        engine.addToOutput("<select name='" + responseIdentifier + "'>");
        
        //process the children
        engine.processElements((List<Content>)(List)children);
    	
        //close the select box
        engine.decreaseDepth();
        engine.addToOutput("</select>", false);
	}
	
	private void processInlineChoice(Element _e){
		//add the option input
        String identifier = _e.getAttribute("identifier").getValue();
    	String temp = "";
        
    	temp = "<option value='" + identifier + "'";
        if(response != null && response.equals(identifier)) temp += " selected='selected'";
        temp += ">";
        engine.addToOutput(temp);
        
        //add the text
        engine.addToOutput(_e.getTextNormalize(), false);
        
        //close option tag
        engine.decreaseDepth();
        engine.addToOutput("</option>", false);
	}

}
