/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.rendering.xhtml.processors;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.jdom.Attribute;
import org.jdom.Content;
import org.jdom.Element;

import r2q2.SpecificationViolationException;
import r2q2.rendering.RenderingException;
import r2q2.rendering.xhtml.UnsupportedTagException;
import r2q2.rendering.xhtml.XHTMLRenderer;
import r2q2.rendering.xhtml.processors.util.ListElements;

public class MatchInteractionProcessor extends InteractionProcessor {
	
	private int maxAssociations = -1;
	private boolean shuffle = false;
	private int processDimension;
	private HashMap<String,Integer> topMatchMax = new HashMap<String, Integer>();
	private HashMap<String,Integer> leftMatchMax = new HashMap<String, Integer>();
	private HashMap<String, Vector<String>> matchGroups = new HashMap<String, Vector<String>>();
	
	public MatchInteractionProcessor(XHTMLRenderer re) throws RenderingException {
		super(re);
		supportedTags.add("matchInteraction");
		supportedTags.add("simpleMatchSet");
		supportedTags.add("simpleAssociableChoice");
		supportedTags.add("prompt");
	}

	@Override
	public void process(Element _e) throws UnsupportedTagException,
			RenderingException, SpecificationViolationException {
		
		String elementName = _e.getName();
		
		//test valid element
		if(!supportedTags.contains(elementName))
			throw new UnsupportedTagException(elementName + "not supported in MatchInteractionProcessor");
		
		if(elementName.equals("matchInteraction")){
			processMatchInteraction(_e);
		}
		else if(elementName.equals("simpleMatchSet")){
			processSimpleMatchSet(_e);
		}
		else if(elementName.equals("simpleAssociableChoice")){
			processSimpleAssociableChoice(_e);
		}
		else if(elementName.equals("prompt")){
			processPrompt(_e);
		}

	}
	
	private void processMatchInteraction(Element _e) throws UnsupportedTagException, RenderingException, SpecificationViolationException{
        List<Attribute> attrs = _e.getAttributes();
        //Now process all of the attributes in the array. 
        String attname = null;
        try{
	        for (Attribute attr: attrs){                   
	            attname = attr.getName();                  
	            String value = attr.getValue();  
	            if(attname.equals("responseIdentifier"))
	                responseIdentifier = value;
	            if(attname.equals("shuffle"))
	                shuffle = Boolean.parseBoolean(value);
	            if(attname.equals("maxAssociations"))
	                maxAssociations = Integer.parseInt(value);
	        }  
        }catch(NumberFormatException nfe)
        {
        	throw new SpecificationViolationException(attname + " could not be converted to the appropriate type");
        }
        
        //set the match interaction
        processDimension = 0;
        
        if(engine.vars == null) System.out.println("engine.vars is null");//TODO delete me
        HashMap<String, HashMap> responseVars = engine.vars[0];
    	HashMap interactionResponse = responseVars.get(responseIdentifier);
    	if(interactionResponse == null)
    		throw new SpecificationViolationException("ResponseVar " + responseIdentifier + " not present");
    	String baseType = (String) interactionResponse.get("baseType");
    	if(!baseType.equals("directedPair"))
    	{
    		//TODO delete
    		System.out.println("response type is : "+baseType);
    		throw new SpecificationViolationException("Cannot assign choices to a variable (" + responseIdentifier + ") that is not of base type directedPair");
    	}
    	String cardinality = (String) interactionResponse.get("cardinality");
    	if(!cardinality.equals("single") && !cardinality.equals("multiple"))
    		throw new SpecificationViolationException("Cannot assign match interaction to a variable (" + responseIdentifier + ") that is not single or multiple cardinality");
    	
        //test that response variable is multiple cardinality if maxChoices !=1
        if(maxAssociations != 1){
	        
	    	if(cardinality.equals("single"))
	    		throw new SpecificationViolationException("Cannot assign multiple choices to a single cardinality variable (" + responseIdentifier + ")");
	    }
        
        System.out.println("values type is " + interactionResponse.get("candidateValues"));
    	response = (Vector) interactionResponse.get("candidateValues");
        
        engine.addToOutput("<table>");
        process(_e.getChildren());
        engine.decreaseDepth();
        engine.addToOutput("</table>",false);
        
        //output the script-call code to set up match limiting
        engine.addToOutput("<script type='text/javascript'>");
        engine.addToOutput("//<![CDATA[");
        
        String temp = "setupMatch('" + responseIdentifier + "', " + maxAssociations + ", {";
        
        Iterator<String> iter = topMatchMax.keySet().iterator();
        while(true){
        	String nextKey = iter.next();
        	Integer nextValue = topMatchMax.get(nextKey);
        	temp += nextKey + ":" + nextValue.toString();
        	if(!iter.hasNext()) break;
        	temp += ",";
        }
        
        temp += "}, {";
        
        iter = leftMatchMax.keySet().iterator();
        while(true){
        	String nextKey = iter.next();
        	Integer nextValue = leftMatchMax.get(nextKey);
        	temp += nextKey + ":" + nextValue.toString();
        	if(!iter.hasNext()) break;
        	temp += ",";
        }
        
        temp += "});";
        engine.addToOutput(temp, false);
        
        engine.decreaseDepth();
        engine.addToOutput("//]]>", false);
        engine.decreaseDepth();
        engine.addToOutput("</script>", false);
        
        
        //if shuffle then randomise the order of non-fixed entries in both matchsets
        
        //create top row using simpleMatchSet1 and prompt if it exists
        //store the identifiers in a simple lookup array
        
        //for the number of data rows create entries of the length of columns
	}
	
	private void processSimpleMatchSet(Element _e)throws UnsupportedTagException, RenderingException, SpecificationViolationException{
		
		//get the choices and shuffle as appropriate
        List<Element> children = _e.getChildren();
        List<List<String>> prevOrderV = engine.opts.get(responseIdentifier);
        if(shuffle){
        	if(prevOrderV == null || prevOrderV.size() <= processDimension){
        		children = ListElements.randomiseList(children);
        		List<String> map = ListElements.getOrder(children);
        		
        		if(prevOrderV == null)
        			prevOrderV = new Vector<List<String>>();
        		prevOrderV.add(map);
        		engine.opts.put(responseIdentifier, prevOrderV);
		    }else
		    {
		    	List<String> orders = prevOrderV.get(processDimension);
		    	children = ListElements.orderList(children, orders);
		    }
        }
        //order now shuffled, move on to actually processing the children
        
        engine.processElements((List)children);
        if(processDimension == 0)
        {
        	engine.decreaseDepth();
			engine.addToOutput("</tr>",false);
        }
        ++processDimension;
	}
	
	private void processSimpleAssociableChoice(Element _e)throws UnsupportedTagException, RenderingException, SpecificationViolationException{
		String id = _e.getAttributeValue("identifier");
		Integer max = new Integer(_e.getAttributeValue("matchMax"));
		
		//store the match groups
		String matchGroup = _e.getAttributeValue("matchGroup");
		if(matchGroup != null){
			String[] groups= matchGroup.split(" ");
			Vector<String> tmp = new Vector<String>();
			for(int i=0; i<groups.length; i++)
				tmp.add(groups[i]);
			matchGroups.put(id, tmp);
		}
		
		if(processDimension == 0)
		{
			topMatchMax.put(id, max);
			engine.addToOutput("<th>");
			engine.processElements(_e.getContent());
			engine.decreaseDepth();
			engine.addToOutput("</th>", false);
		}else
		{
			leftMatchMax.put(id, max);
			engine.addToOutput("<tr>");
			engine.addToOutput("<th>");
			engine.processElements(_e.getContent());
			engine.decreaseDepth();
			engine.addToOutput("</th>",false);
			
			//now add in all the combinations
			List<List<String>> order = engine.opts.get(responseIdentifier);
			for(int i=0; i < order.get(0).size(); i++)
			{
				engine.addToOutput("<td align='center'>");
				
				String otherId = order.get(0).get(i);
				if((matchGroups.get(otherId) != null && matchGroups.get(otherId).contains(id))
						|| (matchGroups.get(id) != null && matchGroups.get(id).contains(otherId))){
					engine.addToOutput("<input name='"+responseIdentifier+"[]' type='checkbox' value ='notAllowed' disabled='true' />", false);
				}
				else{
					boolean checked = false;
					String directedPair = otherId+" "+id;
					if(response != null)
					{
						for(int j=0; j<response.size();j++)
						{
							if(response.get(j).equals(directedPair))
								checked = true;
						}
					}
					engine.addToOutput("<input type='checkbox' name='"+responseIdentifier+"[]' value='"+directedPair+"'"+((checked)?" checked='checked'":"") + " onclick='checkMatch(\"" + responseIdentifier + "\", this)'/>", false);
				}
				
				engine.decreaseDepth();
				engine.addToOutput("</td>",false);
			}
			
			engine.decreaseDepth();
			engine.addToOutput("</tr>", false);
		}
	}
	
	
	private void processPrompt(Element _e)throws UnsupportedTagException, RenderingException, SpecificationViolationException{
		engine.addToOutput("<tr>");
		engine.addToOutput("<th colspan='1' align='center'>");
		engine.processElements(_e.getContent());
		engine.decreaseDepth();
		engine.addToOutput("</th>", false);
	}
}
