/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.rendering.xhtml.processors;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.jdom.Attribute;
import org.jdom.Content;
import org.jdom.DataConversionException;
import org.jdom.Element;

import r2q2.SpecificationViolationException;
import r2q2.rendering.RenderingException;
import r2q2.rendering.xhtml.UnsupportedTagException;
import r2q2.rendering.xhtml.XHTMLRenderer;
import r2q2.rendering.xhtml.processors.util.ListElements;

public class OrderInteractionProcessor extends InteractionProcessor {
	
	private String orientation = "horizontal";
	private boolean shuffle = false;

	public OrderInteractionProcessor(XHTMLRenderer re) throws RenderingException {
		super(re);
		
		supportedTags.add("orderInteraction");
		supportedTags.add("simpleChoice");
	}

	public void process(Element _e) throws UnsupportedTagException,
			RenderingException, SpecificationViolationException {
		
		String elementName = _e.getName();
		
		if(!supportedTags.contains(elementName)){
			throw new UnsupportedTagException(elementName + " in choiceInteraction");
		}
		else if(_e.getName().equals("orderInteraction")){
			processOrderInteraction(_e);
		}
		else if(_e.getName().equals("simpleChoice")){
			processSimpleChoice(_e);
		}
		
	}

	private void processOrderInteraction(Element _e) throws SpecificationViolationException, UnsupportedTagException, RenderingException {

		String attname = ""; 
		try{
			//Now process all of the attributes in the array 
	        Iterator<Attribute> attrs = _e.getAttributes().iterator();
	        while (attrs.hasNext()){          
	            Attribute attr = attrs.next();                  
	            attname = attr.getName();                  
	            String value = attr.getValue();  
	            if(attname.equals("responseIdentifier"))
	                responseIdentifier = value;
	            else if(attname.equals("orientation"))
	            	if(value.equals("horizontal") || value.equals("vertical"))
	            		orientation = value;
	            	else
	            		throw new SpecificationViolationException("Orientation must be either horizontal or vertical");
	            else if(attname.equals("shuffle"))
	                shuffle = Boolean.parseBoolean(value);
	        }
	    }
	    catch(NumberFormatException e){
	    	throw new SpecificationViolationException(attname + " could not be converted to the appropriate type", e);
	    }
        
        //test that response variable is identifier
	    HashMap<String, HashMap> responseVars = engine.vars[0];
    	HashMap interactionResponse = responseVars.get(responseIdentifier);
    	if(interactionResponse == null)
    		throw new SpecificationViolationException("ResponseVar " + responseIdentifier + " not present");
    	String baseType = (String) interactionResponse.get("baseType");
    	if(!baseType.equals("identifier"))
    		throw new SpecificationViolationException("Cannot assign choices to a variable (" + responseIdentifier + ") that is not of base type identifier");
    	String cardinality = (String) interactionResponse.get("cardinality");
    	if(!cardinality.equals("ordered"))
    		throw new SpecificationViolationException("Order interaction must assign to a variable with ordered cardinality");
    	Vector<String> defaultValue = (Vector<String>) interactionResponse.get("defaultValue");
    	
    	response = (Vector) interactionResponse.get("candidateValues");
    	
    	//order the choices
        List<Element> children = _e.getChildren();
        Element prompt = _e.getChild("prompt", engine.ns);
        children.remove(prompt);
        engine.processElement(prompt);
        
        if(children == null || children.size() == 0)
        	throw new SpecificationViolationException("No children of orderInteraction element");
        
        //if necessary, set the order of the choices
        if(response != null){
        	//set the order to the user's response
			List<String> hm = new Vector<String>();
			int i=0;
			System.out.println("response is: " + response);
			for(Element e: children)System.out.println("child: " + e.getAttributeValue("identifier"));
			for(String value: response)
				hm.add(value);
			children = ListElements.orderList(children, hm);
			for(Element e: children)System.out.println("childnow: " + e.getAttributeValue("identifier"));
        }
        else if(defaultValue != null){
        	//set the order to the default value
			List<String> hm = new Vector<String>();
			int i=0;
			for(String value: defaultValue)
				hm.add(value);
			children = ListElements.orderList(children, hm);
        }
        else if(shuffle){
        	List orderList = engine.opts.get(responseIdentifier);
        	if(engine.opts.get(responseIdentifier) == null)
        	{
        		children = ListElements.randomiseList(children);
        		//set the hashtable
        		List<String> map = ListElements.getOrder(children);
        		List<List<String>> mapList= new Vector<List<String>>();
        		mapList.add(map);
        		engine.opts.put(responseIdentifier, mapList);
        	}else
        	{
        		//need to fetch hashtable
        		children = ListElements.orderList(children,(List<String>)orderList.get(0));
        	}
        }
        
        //start the containing <div>
        engine.addToOutput("<div id='" +  responseIdentifier + "'>");
        
        //process the children
        engine.processElements((List<Content>)(List)children);
        
        //close the div and add the javascript
        engine.decreaseDepth();
        engine.addToOutput("</div>", false);
        engine.addToOutput("<br style='clear:left;'/>", false);
        engine.addToOutput("<script type='text/javascript'>");
        engine.addToOutput("//<![CDATA[");
        engine.addToOutput("Sortable.create('" + responseIdentifier + "'," +
        		"{" +
        			"tag:'div'," +
        			"dropOnEmpty:true," +
        			"constraint:'" + orientation + "'," +
        			"overlap:'" + orientation + "'," +
        			"onUpdate:listOrderChange" +
				"})", false);
        engine.decreaseDepth();
        engine.addToOutput("//]]>", false);
        engine.decreaseDepth();
        engine.addToOutput("</script>", false);
	}
	
	private void processSimpleChoice(Element _e) throws UnsupportedTagException, 
						RenderingException, SpecificationViolationException{
		String attempted = "";
		if(response == null) attempted = "Unattempted";
		String identifier = _e.getAttributeValue("identifier");
		
		engine.addToOutput("<div id='_" + identifier + "' " +
								"class='" + orientation + "Draggable" + attempted + "' onmousedown='setAttempted(this.parentNode)'>");
		engine.processElements(_e.getContent());
		engine.decreaseDepth();
		engine.addToOutput("</div>", false);
		if(response != null)
			engine.addToOutput("<input type='hidden' name='" + responseIdentifier +"[]' value='" + identifier + "' />", false);
	}

}
