/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.rendering.xhtml.processors;

import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

import javax.imageio.ImageIO;

import org.jdom.Attribute;
import org.jdom.Element;

import com.gavinwillingham.util.properties.NoSuchPropertyException;
import com.gavinwillingham.util.properties.PropertiesLoader;

import r2q2.SpecificationViolationException;
import r2q2.rendering.RenderingException;
import r2q2.rendering.xhtml.UnsupportedTagException;
import r2q2.rendering.xhtml.XHTMLRenderer;
import r2q2.rendering.xhtml.processors.util.ObjectProcessor;
import r2q2.rendering.xhtml.structure.xhtml.XHTMLObject;
import r2q2.util.strings.TokenUtils;

/**
 * 
 * TODO javadoc
 *
 * @author Gavin Willingham
 *
 */
public class SelectPointInteractionProcessor extends GraphicInteractionProcessor {
	
	private int maxChoices = 1;

	public SelectPointInteractionProcessor(XHTMLRenderer re) throws RenderingException {
		super(re);
		supportedTags.add("selectPointInteraction");
	}

	/**
	 * TODO javadoc
	 */
	public void process(Element child) throws UnsupportedTagException, RenderingException, SpecificationViolationException {
		
		String elementName = child.getName();
		
		//test valid tag
		if(!supportedTags.contains(elementName))
			throw new UnsupportedTagException(elementName + " in selectPointInteraction");
		
		if(elementName.equals("selectPointInteraction"))
			processSelectPointInteraction(child);
	}
	
	private void processSelectPointInteraction(Element _e) throws SpecificationViolationException, RenderingException, UnsupportedTagException{
		
		String attname = "";
		try{
			//Now process all of the attributes in the array 
	        Iterator<Attribute> attrs = _e.getAttributes().iterator();
	        while (attrs.hasNext()){          
	            Attribute attr = attrs.next();                  
	            attname = attr.getName();                  
	            String value = attr.getValue();  
	            if(attname.equals("responseIdentifier"))
	                responseIdentifier = value;
	            else if(attname.equals("maxChoices"))
	                maxChoices = Integer.parseInt(value);
	        }
		}
	    catch(NumberFormatException e){
	    	throw new SpecificationViolationException(attname + " could not be converted to the appropriate type", e);
	    }
        
        //test that response variable is identifier
        HashMap<String, HashMap> responseVars = engine.vars[0];
    	HashMap interactionResponse = responseVars.get(responseIdentifier);
    	if(interactionResponse == null)
    		throw new SpecificationViolationException("ResponseVar " + responseIdentifier + " not present");
    	String baseType = (String) interactionResponse.get("baseType");
    	if(!baseType.equals("point"))
    		throw new SpecificationViolationException("Cannot assign points to a variable (" + responseIdentifier + ") that is not of base type point");
    	
    	//get any previous values, i.e. user response
    	response = (Vector) interactionResponse.get("candidateValues");
    	
    	//output the prompt
    	Element prompt = _e.getChild("prompt", XHTMLRenderer.ns);
    	if(prompt != null) engine.processElement(prompt);
    	
    	//find the object node
    	Element object = _e.getChild("object", XHTMLRenderer.ns);
    	if(object == null)
    		throw new SpecificationViolationException("No object found under SelectPointInteraction");
    	XHTMLObject obj = processObject(object);
    	
    	//set up the applet
        engine.addToOutput("<applet"); 
        engine.addToOutput("codebase='" + basePath + "'", false);
    	engine.addToOutput("code = 'rhotspotV2.class'", false);
    	engine.addToOutput("width = '" + obj.getWidth() + "'", false);
    	engine.addToOutput("height = '" + (obj.getHeight() + 40) + "'", false);
    	engine.addToOutput("hspace = '0'", false);
    	engine.addToOutput("vspace = '0'", false);
    	engine.addToOutput("align = 'middle'", false);
    	engine.addToOutput(">", false);
    	
    	//add the parameters
    	engine.addToOutput("<param name = 'identifier' value = '" + responseIdentifier + "'/>", false);
    	engine.addToOutput("<param name = 'NoOfMainImages' value = '1'/>", false);
    	String imagePath = obj.getData();
    	if(imagePath.startsWith(basePath))
    		imagePath = imagePath.substring(basePath.length());
    	engine.addToOutput("<param name = 'Mainimageno1' value = '" + imagePath + "::0::0::" + obj.getWidth() + "::" + obj.getHeight() + "'/>", false);
    	engine.addToOutput("<param name = 'markerName' value = 'images/marker.gif'/>", false);
    	engine.addToOutput("<param name = 'baseType' value = 'point '/>", false);
    	engine.addToOutput("<param name = 'noOfTargets' value = '0'/>", false);
    	engine.addToOutput("<param name = 'identifiedTargets' value = 'FALSE'/>", false);
    	engine.addToOutput("<param name = 'noOfMarkers' value ='" + maxChoices + "'/>", false);
    	engine.addToOutput("<param name = 'markerType' value ='STANDARD'/>", false);
    	
    	//add previous input, if any
    	String temp = "<param name = 'feedbackState' value = '";
    	if(response != null){
    		temp += "Yes";
    		for(String resp: response){
    			temp += ":" + resp;
    		}
    	}
    	else temp += "No";
    	temp += "'/>";
    	engine.addToOutput(temp, false);
    	
    	//close the applet
    	engine.decreaseDepth();
    	engine.addToOutput("</applet>", false);
	}
}
