/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.rendering.xhtml.processors;

import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.jdom.Attribute;
import org.jdom.Element;

import r2q2.SpecificationViolationException;
import r2q2.rendering.RenderingException;
import r2q2.rendering.xhtml.UnsupportedTagException;
import r2q2.rendering.xhtml.XHTMLRenderer;

public class TextEntryInteractionProcessor extends InteractionProcessor {

	public TextEntryInteractionProcessor(XHTMLRenderer re) throws RenderingException {
		super(re);
		supportedTags.add("textEntryInteraction");
	}

	@Override
	public void process(Element _e) throws UnsupportedTagException,
			RenderingException, SpecificationViolationException {
		
		String elementName = _e.getName();
		
		//test valid element
		if(!supportedTags.contains(elementName))
			throw new UnsupportedTagException(elementName + "not supported in TextEntryInteractionProcessor");
		
		//grab the attributes
        String stringIdentifier = null;
        int expectedLength = 20;
        String ssClass = "";
        String id = "";
        String placeholderText = "";
        String patternMask = null;
        
        String attname = "";
        try{
	        List<Attribute> attrs = _e.getAttributes();
	        //Now process all of the attributes in the array. 
	        for (Attribute attr: attrs){                   
	            attname = attr.getName();                  
	            String value = attr.getValue();  
	            if(attname.equals("responseIdentifier"))
	                responseIdentifier = value;
	            else if(attname.equals("stringIdentifier"))
	                stringIdentifier = value;
	            else if(attname.equals("expectedLength"))
	                expectedLength = Integer.parseInt(value);
	            else if(attname.equals("id"))
	                id = value;
	            else if(attname.equals("class"))
	                ssClass = value;
	            else if(attname.equals("placeholderText"))
	                placeholderText = value;
	            else if(attname.equals("patternMask"))
	            	patternMask = value;
	        }  
        }
	    catch(NumberFormatException e){
	    	throw new SpecificationViolationException(attname + " could not be converted to the appropriate type", e);
	    }
        
        //verify the response identifier
        HashMap<String, HashMap> responseVars = engine.vars[0];
    	HashMap interactionResponse = responseVars.get(responseIdentifier);
    	if(interactionResponse == null)
    		throw new SpecificationViolationException("ResponseVar " + responseIdentifier + " not present");
    	String cardinality = (String) interactionResponse.get("cardinality");
    	if(!cardinality.equals("single"))
    		throw new SpecificationViolationException("ResponseVar " + responseIdentifier + " must have single cardinality for a textEntryInteraction");
    	String baseType = (String) interactionResponse.get("baseType");
    	if(!baseType.equals("string")
    			&& !baseType.equals("integer")
    			&& !baseType.equals("float"))
    		throw new SpecificationViolationException("ResponseVar " + responseIdentifier + " must be a string, integer or float for a textEntryInteraction, is a " + baseType);
        
    	//if present, verify the stringIdentifier
    	if(stringIdentifier != null){
	    	HashMap stringResponse = responseVars.get(responseIdentifier);
	    	if(interactionResponse == null)
	    		throw new SpecificationViolationException("String ResponseVar " + stringIdentifier + " not present");
	    	cardinality = (String) stringResponse.get("cardinality");
	    	if(!cardinality.equals("single"))
	    		throw new SpecificationViolationException("String ResponseVar " + stringIdentifier + " must have single cardinality for a textEntryInteraction");
	    	baseType = (String) stringResponse.get("baseType");
	    	if(!baseType.equals("string"))
	    		throw new SpecificationViolationException("String ResponseVar " + stringIdentifier + " must be a string for a textEntryInteraction");
    	}
    	
    	//generate the content check
    	String contentCheck = "";
    	if(baseType.equals("float"))
    		contentCheck += "checkFloat(this)";
    	else if (baseType.equals("integer"))
    		contentCheck += "checkInteger(this)";
    	if(patternMask != null)
    		contentCheck += " checkRegex(this, " + patternMask + ");";
    	
        //build the interaction
        String temp ="<input type=text name='" + responseIdentifier;
        if(stringIdentifier != null) temp += ":" + stringIdentifier;
        temp += "' size=" + expectedLength + "' value='";
        
        //add in the user's response
        HashMap var = (HashMap) engine.vars[0].get(responseIdentifier);
        String value = null;
        Vector values = (Vector)var.get("candidateValues");
        if(values != null) value = (String) values.get(0);
        if(value != null) temp += value;
        else temp += placeholderText;
        
        //add in the content check
        if(!contentCheck.equals(""))
        	temp += "' onchange='" + contentCheck;
        
        temp +=	"' />";
        
        engine.addToOutput(temp, false);
	}

}
