/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.rendering.xhtml.processors.util;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashSet;
import java.util.List;

import javax.imageio.ImageIO;

import org.jdom.Element;

import r2q2.SpecificationViolationException;
import r2q2.rendering.RenderingException;
import r2q2.rendering.xhtml.XHTMLRenderer;
import r2q2.rendering.xhtml.structure.xhtml.XHTMLObject;

/**
 * 
 * TODO javadoc
 *
 * @author Gavin Willingham
 *
 */
public class ObjectProcessor {
	
	private HashSet<String> supportedObjectTypes;
	
	public ObjectProcessor(HashSet<String> _supportedObjectTypes){
		this.supportedObjectTypes = _supportedObjectTypes;
	}

	public XHTMLObject processObject(Element _e) throws RenderingException, SpecificationViolationException{
		//extract the attributes
		String data = _e.getAttributeValue("data");
		if(data == null) 
			throw new SpecificationViolationException("'data' attribute not set in object");
		String type = _e.getAttributeValue("type");
		if(type == null) 
			throw new SpecificationViolationException("'type' attribute not set in object");
		if(!supportedObjectTypes.contains(type)){
			
			List<Element> children = _e.getChildren();
			
			if(children.size() != 1 || !children.get(0).getName().equals("object"))
				throw new RenderingException("Unsupported object type");
			
			return processObject(children.get(0));
		}
		else{
			XHTMLObject obj_ = new XHTMLObject();
			obj_.setData(data);
			obj_.setType(type);
			
			int width = -1;
			int height = -1;
			
			String strWidth = _e.getAttributeValue("width");
			try{
				if(strWidth != null) width = Integer.parseInt(strWidth);
				else System.out.println("pie");
			}
			catch(NumberFormatException e){
				throw new SpecificationViolationException("Width not an integer");
			}
			String strHeight = _e.getAttributeValue("height");
			try{
				if(strHeight != null) height = Integer.parseInt(strHeight);
			}
			catch(NumberFormatException e){
				throw new SpecificationViolationException("Height not an integer");
			}
			
			if(width == -1 || height == -1){
				BufferedImage img = null;
				try {
					img = ImageIO.read(new URL(data));
				} catch (MalformedURLException e) {
					throw new RenderingException("Invalid URL to remote image file", e);
				} catch (IOException e) {
					throw new RenderingException("Couldn't read remote image file", e);
				}
				if(width == -1) width = img.getWidth();
				if(height == -1) height = img.getHeight();
			}
			
			obj_.setHeight(height);
			obj_.setWidth(width);
			
			return obj_;
		}
	}
}
