/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.util.xml;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.xml.sax.InputSource;

import com.gavinwillingham.util.properties.NoSuchPropertyException;
import com.gavinwillingham.util.properties.PropertiesLoader;

public class DocumentBuilder {

	public static Element getRootElement(byte[] _xml) throws JDOMException, IOException, NoSuchPropertyException{
		
		Document doc = getDocument(_xml);
	    return (Element) doc.getRootElement();
	}

	public static Document getDocument(byte[] _xml) throws IOException, NoSuchPropertyException, JDOMException {
//		Initalise Sax builder
	    SAXBuilder builder = new SAXBuilder();
	    builder.setFeature("http://xml.org/sax/features/validation", false); 
	    builder.setFeature("http://apache.org/xml/features/validation/schema", false);
	    
	    //set the schema location via PropertiesLoader
	    String xsd = PropertiesLoader.getProperty("r2q2.xml.xsdLocation");
	    builder.setProperty("http://apache.org/xml/properties/schema/external-schemaLocation",
	    	"http://www.imsglobal.org/xsd/imsqti_v2p0 " + xsd);
	    builder.setProperty("http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation",
			xsd);
	       
	    //create document object
	    Document doc;
	    doc = builder.build(
	    		new InputSource(
	    			new ByteArrayInputStream(_xml)));
	    
	    return doc;
	}

}
