/*
<LICENCE>

Copyright (c) 2007, University of Southampton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

  *	Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

  *	Neither the name of the University of Southampton nor the names of its
	contributors may be used to endorse or promote products derived from this
	software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

</LICENCE>
*/

package r2q2.variable;

import java.util.HashMap;
import java.util.Vector;

public abstract class ItemVariable {
	public enum Cardinality {single, multiple, ordered, record}
	public enum BaseType {bool,coords,duration,file,afloat,identifier,integer,
        				  string,uri,point,pair,directedPair,shape}
	
	public enum Shape {defaults,rect,circle,poly,ellipse}
	
	public Object defaultValue;
	public String defaultValueInterpretation;
	
	public String identifier;
	public BaseType varType;
	public Cardinality card;
	
	public Object value;
	
	public abstract Object getDefaultValue();
	
	public HashMap<String,Object> toLegacyFormat()
	{
		HashMap<String,Object> vals = new HashMap<String,Object>();
		switch(card)
		{
			case single: vals.put("cardinality", "single");break;
			case multiple: vals.put("cardinality", "multiple");break;
			case ordered: vals.put("cardinality", "ordered");break;
			case record: vals.put("cardinality", "record");break;
		}
		switch(varType)
		{
			case bool: vals.put("baseType", "boolean");break;
			case coords: vals.put("baseType", "coords");break;
			case duration: vals.put("baseType", "duration");break;
			case file: vals.put("baseType", "file");break;
			case afloat: vals.put("baseType", "float");break;
			case identifier: vals.put("baseType", "identifier");break;
			case integer: vals.put("baseType", "integer");break;
			case string: vals.put("baseType", "string");break;
			case uri: vals.put("baseType", "uri");break;
			case point: vals.put("baseType", "point");break;
			case pair: vals.put("baseType", "pair");break;
			case directedPair: vals.put("baseType", "directedPair");break;
		}
		
		if(defaultValue != null)
		{
			Vector<String> strForm = new Vector<String>();
			if(this.defaultValue instanceof Vector)
			{
				for(int i=0; i < ((Vector)defaultValue).size(); i++)
					strForm.add(((Vector)defaultValue).elementAt(i).toString());
			}
			else
			{
				if(defaultValue != null)
					strForm.add(defaultValue.toString());
			}
			if(strForm.size() > 0)
				vals.put("defaultValue", strForm);
			else
				vals.put("defaultValue", null);
		}
		return vals;
	}
	
	public boolean equals(ItemVariable compare)
	{
		if(compare.varType == this.varType &&
		   compare.value == this.value 	   &&
		   compare.card == this.card)
			return true;
		return false;
	}
}
